page	60,132
title	(C) 1991 Computone Corporation -- fifob.asm

;******************************************************************************
;*                                                                            *
;*                                                                            *
;*                                                                            *
;*                  (C) Copyright 1991 Computone Corporation                  *
;*                                                                            *
;*          Unauthorized use, duplication, or disclosure prohibited           *
;*                                                                            *
;*              This module contains PROPRIETARY INFORMATION of               *
;*                            Computone Corporation                           *
;*                   and should be treated as CONFIDENTIAL.                   *
;*                                                                            *
;*                                                                            *
;*                                                                            *
;******************************************************************************

VERSION		equ	1
REVISION	equ	0
SUBREVISION	equ	1

;----------------------------------------------------------------
; fifob.asm
; Supports the "Extensive loopback test"
; Variable sized blocks are looped back, and the
; mailbox registers are used to synchronize between
; the host and board (to signal when data has been read,
; is ready to be read, etc.)
;
; The goal is to get lots of stuff happening at once, for example,
; both the host and board may be reading the fifo at the same time
; (or writing it).
;
; This module uses the fifo interface in its native mode: performs
; 16-bit transfers if configured to do so, otherwise 8-bit.
;------------------------------------------------------------------ 

; 
;----------------------------------------------------------------
; Revision history:
;
;1.0.0 -- 6/6/91 --		first edition
;1.0.1 -- 10/29/91	MAG	First mods to support native-mode
;				loopback support.
;----------------------------------------------------------------

include lw_h1.inc


buffer db 256 dup(0)

maskvalue  db 0

;-- Long-term register use:
;	bl = led color
;	bh = mode (expandable, 16/8-bit mode)

initialization:

	mov	bl, YELLOW
	TOGGLE_LED	bl	; Toggles to green before writing

	cli
	cld

	;--- Determine whether we are in 16-bit or 8-bit mode
	xor	ax, ax
	mov	ds, ax

	GET_OPERATING_MODE
	mov	bh, al

ASSUME cs:_TEXT, ds:_TEXT

	mov	ax,cs		;set all seg registers to the code space
	mov	ds,ax
	mov	es,ax
	mov	ss,ax

	;Block-oriented "local" fifo loopback
	;no stack used
	; LED color toggles after each block is transferred

	mov	maskvalue, 0
	WRITE_MASK_VALUE	bh, 0

	;----------------------------
	; -- Wait for a mailbox hit--
	;----------------------------

wait_1:
	WAIT_FOR_MAILBOX_IN	bh

	;----------------------------------
	; -- Read the first block from fifo
	;----------------------------------

	; The test protocol ensures there will be only one
	; block here: therefore the first test should find the
	; fifo non-empty, the second test empty.
	; This practically ensures test failure if the empty
	; bit doesn't track properly.

repeat_1:
	mov	dx, FIFO_CMD
	in	al, dx		; read the status register
	test	al, EMPT
	jnz	done_1

	mov	dx, FIFO_DATA
	mov	di, offset buffer
	mov	si, di

	test	bh, IS_16
	jnz	move_1

	in	al, dx
	mov	cl, al
	in	al, dx
	mov	ch, 0
	mov	bp, cx	; save the size

	rep	insb
	jmp	repeat_1

move_1:
	in	ax, dx	; get the size
	xor	ah, ah
	mov	cx, ax
	mov	bp, cx

	shr	cx, 1
	rep	insw
	jmp	repeat_1
	
	;-----------------------
	; -- Hit the mailbox  --
	;-----------------------

done_1:
	HIT_MAILBOX	bh, 10h

write_1:
	;-----------------------------------
	; -- Write the data back to fifo --
	;------------------------------------
	mov	dx, FIFO_DATA
	mov	cx, bp
	mov	ax, cx

	test	bh, IS_16
	jnz	write_1a

	out	dx, al
	mov	al, ah
	out	dx, al
	rep	outsb
	jmp	hit_2

write_1a:
	out	dx, ax	; write the count
	shr	cx, 1
	rep	outsw

	;-------------------------------------------------
	; -- Hit the mailbox, once host has read last mail
	; -- Then wait for a mailbox hit
	;-------------------------------------------------
hit_2:
	WAIT_FOR_MB_READ	bh
	HIT_MAILBOX		bh, 20h
	WAIT_FOR_MAILBOX_IN	bh

repeat_2:
	;----------------------------
	; -- Read the second block --
	;----------------------------

	mov	dx, FIFO_CMD
	in	al, dx		; read the status register
	test	al, EMPT
	jnz	done_2

	mov	di, offset buffer
	mov	si, di
	mov	dx, FIFO_DATA

	test	bh, IS_16
	jnz	move_2

	in	al, dx
	mov	cl, al
	in	al, dx
	mov	ch, 0
	mov	bp, cx	; save the size

	rep	insb
	jmp	repeat_2

move_2:
	in	ax, dx
	xor	ah, ah
	mov	cx, ax
	mov	bp, cx

	shr	cx, 1
	rep	insw
	jmp	repeat_2

done_2:

	;-----------------------------------
	; -- Write the data back to fifo --
	;------------------------------------

	mov	dx, FIFO_DATA
	mov	cx, bp
	mov	ax, cx

	test	bh, IS_16
	jnz	write_2

	out	dx, al
	mov	al, ah
	out	dx, al
	rep	outsb
	jmp	done_3

write_2:
	out	dx, ax
	shr	cx, 1
	rep	outsw

	;-------------------------------------------------
	; -- Hit the mailbox, once host has read last mail
	;-------------------------------------------------
done_3:
	WAIT_FOR_MB_READ	bh
	HIT_MAILBOX		bh, 30h


	;----------------------------------------------
	; Change the value in the mask register in case
	; it might affect the integrity of the data
	;----------------------------------------------

	inc	maskvalue
	WRITE_MASK_VALUE	bh, maskvalue

	;---------------------
	; -- Toggle the LED --
	;---------------------

	TOGGLE_LED	bl
	jmp	wait_1
	
include lw_h2.inc
