page	60,132
title	(C) 1991 Computone Corporation -- fifob.asm

;******************************************************************************
;*                                                                            *
;*                                                                            *
;*                                                                            *
;*                  (C) Copyright 1991 Computone Corporation                  *
;*                                                                            *
;*          Unauthorized use, duplication, or disclosure prohibited           *
;*                                                                            *
;*              This module contains PROPRIETARY INFORMATION of               *
;*                            Computone Corporation                           *
;*                   and should be treated as CONFIDENTIAL.                   *
;*                                                                            *
;*                                                                            *
;*                                                                            *
;******************************************************************************

VERSION		equ	1
REVISION	equ	0
SUBREVISION	equ	2

;----------------------------------------------------------------
; fifod.asm
; Supports the "FIFO DMA loopback test"
; 
; For each pass, we wait for a mailbox hit (value don't care)
; Then, under DMA control, we write 16k words (or bytes) to the FIFO
; and read 16k (or bytes) into another buffer.
; Then we compare.
; If all ok, write 0 to mailbox.
; If timeout on write, write 10h to mailbox
; If timeout on read, write 20h to mailbox
; If mismatching data, write 30h to mailbox.
;
; Writes bytes or words accordingo to native mode.
;
; Note, this is only supported by the IntelliPort-IIEX boards
;------------------------------------------------------------------ 

; 
;----------------------------------------------------------------
; Revision history:
;
;1.0.1 -- 11/18/91	MAG	First Draft.
;1.0.2 -- 12/17/91  MAG Bug fixed:  was always running in 16-bit mode!
;
;----------------------------------------------------------------

include lw_h1.inc


mode	db	0
color	db 	0

source_mem_high	dw	0
source_mem_low	dw	0
dest_mem_high	dw	0
dest_mem_low	dw	0
buff_mem_seg	dw	0
read_ctrl	dw	0
write_ctrl	dw	0


;-- Memory usage for buffers:
;	10000 = (1000:0) source buffer
;	18000 = (1800:0) destination buffer 


;-- Initialization phase buffer use...
;	bl = led color
;	bh = mode (expandable, 16/8-bit mode)

initialization:

	mov	bl, YELLOW
	TOGGLE_LED	bl	; Toggles to green before writing

	cli
	cld

	;--- Determine whether we are in 16-bit or 8-bit mode
	xor	ax, ax
	mov	ds, ax

	GET_OPERATING_MODE
	mov	bh, al

ASSUME cs:_TEXT, ds:_TEXT

	mov	ax,cs		;set all seg registers to the code space
	mov	ds,ax
	mov	es,ax
	mov	ss,ax

	mov	color, bl
	mov	mode, bh

	;--- Initialize a block of test data:

	mov	ax, 1000h
	mov	es, ax
	mov	cx, 16 * 1024
	xor	ax, ax
	xor	di, di
ir0:
	stosw		; store the value
	inc	ax
	loop	ir0	; increment, and loop

	mov	ax, cs
	mov	es, ax

	mov	source_mem_high, 1
	mov	source_mem_low, 0
	mov	dest_mem_high, 1
	mov	dest_mem_low, 8000h
	mov	buff_mem_seg, 1000h

	;-- Set up the control word for reading: let this be a higher priority
	;-- than writing.  
	mov	ax, DC_START or DC_SOURCE_SYNC or DC_TERMINATE or DC_DMEM or DC_DINC or DC_PRI
	test	mode, IS_16
	jz	ir1
	or	ax, DC_WORD
ir1:
	mov	read_ctrl, ax
	mov	ax, DC_START or DC_DEST_SYNC or DC_TERMINATE or DC_SMEM or DC_SINC
	test	mode, IS_16
	jz	ir2
	or	ax, DC_WORD
ir2:
	mov	write_ctrl, ax

	;----------------------------
	; -- Wait for a mailbox hit--
	;----------------------------

wait_1:
	WAIT_FOR_MAILBOX_IN	mode

	;------------------------------------------
	; Prepare DMA to read first block from FIFO
	;------------------------------------------

	mov	dx, DMA1_DESTL
	mov	ax, dest_mem_low
	out	dx, ax

	mov	dx, DMA1_DESTH
	mov	ax, dest_mem_high
	out	dx, ax

	mov	dx, DMA1_COUNT
	mov	ax, 16 * 1024
	out	dx, ax

	mov	dx, DMA1_SOURCEL
	mov	ax, FIFO_DATA
	out	dx, ax

	mov	dx, DMA1_SOURCEH
	xor	ax, ax
	out	dx, ax

	mov	dx, DMA1_CTRL
	mov	ax, read_ctrl
	out	dx, ax

	;------------------------------------------
	; Prepare DMA and write block to FIFO
	;------------------------------------------

	mov	dx, DMA0_SOURCEL
	mov	ax, source_mem_low
	out	dx, ax

	mov	dx, DMA0_SOURCEH
	mov	ax, source_mem_high
	out	dx, ax

	mov	dx, DMA0_COUNT
	mov	ax, 16 * 1024
	out	dx, ax

	mov	dx, DMA0_DESTL
	mov	ax, FIFO_DATA
	out	dx, ax

	mov	dx, DMA0_DESTH
	xor	ax, ax
	out	dx, ax

	mov	dx, DMA0_CTRL
	mov	ax, write_ctrl
	out	dx, ax

	;----------------------------------
	; Wait until all the transfers have
	; completed , but only up to a certain
	; limit.
	;----------------------------------

	;----------------------------------
	; Allow at most a couple seconds for
	; the host to read all the data.
	;
	; The local processor runs at 20MHz,
	; the inner loop needs about 5 reads 
	; or 20 bus cycles, or 1uS total.
	; This times 10000 will run 10Ms, 
	; times 100 would be a second.
	; let it be 200 for safety.  Actually,
	; the host should be MUCH faster than 
	; this!.
	;----------------------------------

	mov	dx, DMA0_CTRL
	mov	bx, 200			
wait_2a:
	mov	cx, 10000
wait_2:
	in	ax, dx
	test	ax, DC_START
	jz	done_2
	loop	wait_2

	dec	bx
	jnz	wait_2a
	jmp	bad_out_time

	;----------------------------------
	; Once we know the data was all 
	; written, the host should have
	; written it all back (since he
	; loops back character-by-character).
	; So this timeout should not be necessary
	; except to allow for the time needed for
	; the incoming DMA channel to read in
	; the data.
	;------------------------------------
done_2:
	mov	dx, DMA1_CTRL
	mov	bx, 100
wait_3a:
	mov	cx, 10000	
wait_3:
	in	ax, dx
	test	ax, DC_START
	jz	done_3
	loop	wait_3		; Wait until the reads terminate.

	dec	bx
	jnz	wait_3a
	jmp	bad_in_time

	;------------------------------------
	; Compare
	;------------------------------------

done_3:

	mov	ax, ss:buff_mem_seg
	mov	ds, ax
	mov	es, ax
	mov	si, 0
	mov	di, 8000h
	mov	cx, 16 * 1024

	test	ss:mode, IS_16
	jnz	c16

	shr	cx, 1	; Half as many words, if were transferring bytes

c16:
	repe	cmpsw
	jne	bad_comp

	mov	ax, cs
	mov	ds, ax
	mov	es, ax

	HIT_MAILBOX	mode, 0	; Indicate success (lie, for now!)

	;---------------------
	; -- Toggle the LED --
	;---------------------

	TOGGLE_LED	color
	jmp	wait_1

bad_comp:
	mov	ax, cs
	mov	ds, ax
	mov	es, ax

	HIT_MAILBOX	mode, 30h	; Indicates mis-matched data.
	DEAD_RED			; Indicates failure.

bad_out_time:
	mov	ax, cs
	mov	ds, ax
	mov	es, ax

	HIT_MAILBOX	mode, 10h	; Indicates time-out waiting for
					; data to all be written.
	DEAD_RED			; Indicates failure.

bad_in_time:
	mov	ax, cs
	mov	ds, ax
	mov	es, ax

	HIT_MAILBOX	mode, 20h	; Indicates time-out waiting for
					; data to all be read.
	DEAD_RED			; Indicates failure.

include lw_h2.inc
