
/*---------------------------------------------
 * i2cmd.c
 *
 * IntelliPort-II and IntelliPort-IIEX
 * Definition table for In-line
 * and Bypass commands
 * 
 * Applicable only when the standard loadware
 * is active.
 *
 *---------------------------------------------
 */

/*-------------------------------------
 * Cosmetic note:
 * Written using tabstops at 4 spaces
 * vi: set ts=4
 * pr: -e4
 *-------------------------------------
 */

/*--------------------------------------------
 *
 * Revision History:
 *
 * 10 October 1991	MAG	First Draft
 *  7 November 1991 MAG	Reflects additional commands.
 * 24 February 1992 MAG Additional commands for 1.4.x loadware
 * 11 March 1992	MAG	Additional commands
 * 30 March 1992	MAG Additional command: CMD_DSS_NOW
 * 18 May 1992		MAG	Discovered commands 39 & 40 must be at the
 *					end of a packet: affects implementation.
 *
 *--------------------------------------------
 */

#include "i2cmd.h"	/* To get some bit-defines */

/*------------------------------------------------
 * Here is the table of global arrays which 
 * represent each type of command supported
 * in the IntelliPort-II and -IIEX standard 
 * loadware.
 * See also i2cmd.h for a more complete explanation
 * of what is going on.
 *--------------------------------------------------
 */ 


/* Here are the various globals: note that the names are
 * not used except through the macros defined in i2cmd.h.
 * Also note that although they are character arrays 
 * here (for extendability) they are cast to structure 
 * pointers in the i2cmd.h macros.
 * -- See i2cmd.h for flags definitions.
 */

/*					Length	Flags	Command */	

unsigned char ct02[] ={	1,	BTH,		2		};	/* DTR UP */
unsigned char ct03[] ={ 1,	BTH,		3		};	/* DTR DN */
unsigned char ct04[] ={ 1,	BTH,		4		};	/* RTS UP */
unsigned char ct05[] ={ 1,	BTH,		5		};	/* RTS DN */
unsigned char ct06[] ={ 1,	BTH,		6		};	/* START FL */
unsigned char ct07[] ={ 2,	INL,		7,0		};	/* BAUD */
unsigned char ct08[] ={ 2,	INL,		8,0		};	/* BITS */
unsigned char ct09[] ={ 2,	INL,		9,0		};	/* STOP */
unsigned char ct10[] ={ 2,	INL,		10,0	};	/* PARITY */
unsigned char ct11[] ={ 2,	INL,		11,0	};	/* XON */
unsigned char ct12[] ={ 2,	INL,		12,0	};	/* XOFF */
unsigned char ct13[] ={ 1,	INL,		13		};	/* STOP FL */
unsigned char ct14[] ={ 1,	BYP|VIP,	14		};	/* ACK HOTK */
unsigned char ct15[] ={ 2,	INL|VIP,	15,0	};	/* IRQ SET */
unsigned char ct16[] ={ 2,	BTH,		16,0	};	/* IXONOPTS */
unsigned char ct17[] ={ 2,	INL,		17,0	};	/* OXONOPTS */
unsigned char ct18[] ={ 1,	BTH,		18		};	/* CTSENAB */
unsigned char ct19[] ={ 1,	INL,		19		};	/* CTSDSAB */
unsigned char ct20[] ={ 1,	BTH,		20		};	/* DCDENAB */
unsigned char ct21[] ={ 1,	INL,		21		};	/* DCDDSAB */
unsigned char ct22[] ={ 1,	INL,		22		};	/* DSRENAB */
unsigned char ct23[] ={ 1,	INL,		23		};	/* DSRDSAB */
unsigned char ct24[] ={ 1,	INL,		24		};	/* RIENAB */
unsigned char ct25[] ={ 1,	INL,		25		};	/* RIDSAB */
unsigned char ct26[] ={ 2,	INL,		26,0	};	/* BRKENAB */
unsigned char ct27[] ={ 1,	INL,		27		};	/* BRKDSAB */
unsigned char ct28[] ={ 2,	BTH,		28,0	};	/* MAXBLOKSIZE */
unsigned char ct29[] ={ 2,	0,			29,0	};	/* reserved */
unsigned char ct30[] ={ 1,	INL,		30		};	/* CTSFLOWENAB */
unsigned char ct31[] ={ 1,	INL,		31		};	/* CTSFLOWDSAB */
unsigned char ct32[] ={ 1,	INL,		32		};	/* RTSFLOWENAB */
unsigned char ct33[] ={ 1,	INL,		33		};	/* RTSFLOWDSAB */
unsigned char ct34[] ={ 2,	INL,		34,0	};	/* ISTRIPMODE */
unsigned char ct35[] ={ 2,	INL | END,	35,0	};	/* SENDBREAK */
unsigned char ct36[] ={ 2,	INL,		36,0	};	/* SETERRMODE */
unsigned char ct36a[]={ 3,	INL,		36,0,0	};	/* SET_REPLACEMENT */

/* The following is listed for completeness, but should never
 * be sent directly by user-level code.  It is sent only by library
 * routines in response to data movement.
 */

unsigned char ct37[] ={ 5,	BYP|VIP,	37,0,0,0,0 };
													/* FLOW PACKET */

/* Back to normal */
unsigned char ct38[] ={ 11,	INL|VAR,	38,0,0,0,0,0,0,0,0,0,0 };
													/* DEFINE KEY SEQ */
unsigned char ct39[] ={ 3,	INL | END,	39,0,0	};	/* OPOSTON */
unsigned char ct40[] ={ 1,	INL | END,	40		};	/* OPOSTOFF */
unsigned char ct41[] ={ 1,	BYP,		41		};	/* RESUME */
unsigned char ct42[] ={ 2,	INL,		42,0	};	/* TXBAUD */
unsigned char ct43[] ={ 2,	INL,		43,0	};	/* RXBAUD */
unsigned char ct44[] ={ 2,	INL,		44,0	};	/* MS PING */
unsigned char ct45[] ={ 1,	INL,		45		};	/* HOTENAB */
unsigned char ct46[] ={ 1,	INL,		46		};	/* HOTDSAB */
unsigned char ct47[] ={ 7,	INL,		47,0,0,0,0,0,0};
													/* UNIX FLAGS */
unsigned char ct48[] ={ 1,	INL,		48		};	/* DSRFLOWENAB */
unsigned char ct49[] ={ 1,	INL,		49		};	/* DSRFLOWDSAB */
unsigned char ct50[] ={ 1,	INL,		50		};	/* DTRFLOWENAB */
unsigned char ct51[] ={ 1,	INL,		51		};	/* DTRFLOWDSAB */
unsigned char ct52[] ={ 1,	INL,		52		};	/* BAUDTABRESET */
unsigned char ct53[] ={ 3,	INL,		53,0,0	};	/* BAUDREMAP */
unsigned char ct54[] ={ 3,	INL,		54,0,0	};	/* CUSTOMBAUD1 */
unsigned char ct55[] ={ 3,	INL,		55,0,0	};	/* CUSTOMBAUD2 */
unsigned char ct56[] ={ 2,	INL|END,	56,0	};	/* PAUSE */
unsigned char ct57[] ={ 1,	BYP,		57		};	/* SUSPEND */
unsigned char ct58[] ={ 1,	BYP,		58		};	/* UNSUSPEND */
unsigned char ct59[] ={ 2,	INL,		59,0	};	/* PARITYCHK */
unsigned char ct60[] ={ 1,	INL|VIP,	60		};	/* BOOKMARKREQ */
unsigned char ct61[] ={ 2,	INL,		61,0	};	/* INTERNALLOOP */
unsigned char ct62[] ={ 2,	INL,		62,0	};	/* HOTKTIMEOUT */
unsigned char ct63[] ={ 2,	INL,		63,0	};	/* SETTXON */
unsigned char ct64[] ={ 2,	INL,		64,0	};	/* SETTXOFF */
unsigned char ct65[] ={ 2,	INL,		65,0	};	/* SETAUTORTS */
unsigned char ct66[] ={ 2,	INL,		66,0	};	/* SETHIGHWAT */
unsigned char ct67[] ={ 2,	BYP,		67,0	};	/* STARTSELFL */
unsigned char ct68[] ={ 2,	INL,		68,0	};	/* ENDSELFL */
unsigned char ct69[] ={ 1,	BYP,		69		};	/* HWFLOW_OFF */
unsigned char ct70[] ={ 1,	INL,		70		};	/* ODSRFL_ENAB */
unsigned char ct71[] ={ 1,	INL,		71		};	/* ODSRFL_DSAB */
unsigned char ct72[] ={ 1,	INL,		72		};	/* ODCDFL_ENAB */
unsigned char ct73[] ={ 1,	INL,		73		};	/* ODCDFL_DSAB */
unsigned char ct74[] ={ 2,	INL,		74,0	};	/* LOADLEVEL */
unsigned char ct75[] ={ 2,	INL,		75,0	};	/* STATDATA */
unsigned char ct76[] ={ 1,	BYP,		76		};	/* BREAK_ON */
unsigned char ct77[] ={ 1,	BYP,		77		};	/* BREAK_OFF */
unsigned char ct78[] ={ 1,	BYP,		78		};	/* GETFC */
unsigned char ct79[] ={ 2,	BYP,		79,0	};	/* XMIT_NOW */
unsigned char ct80[] ={ 4,	INL,		80,0,0,0};	/* DIVISOR_LATCH */
unsigned char ct81[] ={ 1,	BYP,		81		};	/* GET_STATUS */
unsigned char ct82[] ={ 1,	BYP,		82		};	/* GET_TXCNT */
unsigned char ct83[] ={ 1,	BYP,		83		};	/* GET_RXCNT */
unsigned char ct84[] ={ 1,	BYP,		84		};	/* GET_BOXIDS */
unsigned char ct85[] ={ 10,	BYP,		85,0,0,0,0,0,0,0,0,0
												};	/* ENAB_MULT */
unsigned char ct86[] ={ 2,	BTH,		86,0	};	/* RCV_ENABLE */
unsigned char ct87[] ={ 1,	BYP,		87		};	/* HW_TEST */
unsigned char ct88[] ={ 3,	INL,		88,0,0	};	/* RCV_THRESHOLD */
unsigned char ct89[] ={ 1,	BYP,		89		};	/* DSS_NOW */


/* Some composite commands as well */

unsigned char cc01[] ={	2,	BTH,		2,4		};	/* DTR & RTS UP */
unsigned char cc02[] ={ 2,	BTH,		3,5		};	/* DTR & RTS DN */



/*------------------------------------------------------
 * i2cmdSetSeq(type, size, string)
 *
 * Function to set the parameters of command 38 Define Hot Key
 * sequence (alias "special receive sequence")
 * Returns a pointer to the structure.
 * Endeavoes to be bullet-proof in that the sequence number
 * is forced in range, and any out-of-range sizes are
 * forced to zero.
 *------------------------------------------------------
 */

cmdSyntaxPtr 
i2cmdSetSeq(unsigned char type, unsigned char size, unsigned char *string)
{
register cmdSyntaxPtr pCM = (cmdSyntaxPtr) ct38;
register unsigned char *pc;

	pCM->cmd[1] = ((type > 0xf) ? 0xf : type);	/* Sequence number */
	size = ((size > 0x8) ? 0 : size);			/* size */
	pCM->cmd[2] = size;
	pCM->length = 3 + size;		/* UPDATES THE LENGTH! */

	pc = &(pCM->cmd[3]);

	while(size--)
	{
		*pc++ = *string++;
	}
	return pCM;
}
	


/*-----------------------------------------------------
 * i2cmdUnixFlags(iflag, oflag, cflag, lflag)
 * 
 * Function to set the parameters of a command 47,
 * then return a pointer to the appropriate structure.
 *-----------------------------------------------------
 */

cmdSyntaxPtr
i2cmdUnixFlags(	unsigned short iflag,
				unsigned short cflag,
				unsigned short lflag)
{
register cmdSyntaxPtr pCM = (cmdSyntaxPtr) ct47;

	pCM->cmd[1] = (unsigned char)  iflag;
	pCM->cmd[2] = (unsigned char) (iflag >> 8);
	pCM->cmd[3] = (unsigned char)  cflag;
	pCM->cmd[4] = (unsigned char) (cflag >> 8);
	pCM->cmd[5] = (unsigned char)  lflag;
	pCM->cmd[6] = (unsigned char) (lflag >> 8);
	return pCM;
}


/*---------------------------------------------------
 * i2cmdBaudRemap(dest,src)
 *
 * Function to set up the parameters of a command 53
 * then return a pointer to the appropriate structure
 *---------------------------------------------------
 */

cmdSyntaxPtr
i2cmdBaudRemap(unsigned char dest, unsigned char src)
{
register cmdSyntaxPtr pCM = (cmdSyntaxPtr) ct53;

	pCM->cmd[1] = dest;
	pCM->cmd[2] = src;
	return pCM;
}



/*------------------------------------------------
 * i2cmdBaudDef(which, rate)
 *
 * Function to set up the parameters of a command
 * 54 or 55 (according to the argument which), then
 * return a pointer to the appropriate structure.
 *------------------------------------------------
 */


cmdSyntaxPtr
i2cmdBaudDef(int which, unsigned short rate)
{
register cmdSyntaxPtr pCM;

	switch(which)
	{
	case 1:	pCM = (cmdSyntaxPtr) ct54;
			break;

	default:
	case 2:	pCM = (cmdSyntaxPtr) ct55;
			break;
	}
	pCM->cmd[1] = (unsigned char) rate;
	pCM->cmd[2] = (unsigned char) (rate >> 8);
	return pCM;
}
	
