//i2dos.c -- C-portion of dos device driver for IntelliPort-II boards

#define VERSION "0.01"

//0.01 -- first beta release 4/29/91
//0.00 -- began 4/15/91

extern char i2_version[];

//compile with Borland C++ (or similar) with the following:
//	bcc -w -1 -mt -f- -a- -B -c i2dosc.c

//All routines are in alphabetical order (except for the "strategy" and
// "interrupt" routines, which precede everything else).

//If the spacing looks a little goofy, try tabs every 4 columns

//the following can be used anywhere for debugging output
void xdebug (char *, int);
void xputs (char *);

typedef unsigned char uchar;
typedef unsigned short ushort;
typedef unsigned long ulong;

#define FP_SEG( fp )( (unsigned )( void _seg * )( void far * )( fp ))
#define FP_OFF( fp )( (unsigned )( fp ))

#define inb inportb
#define outb outportb

#include "i2glob.h"
	//mostly function prototypes for the library

#define MAX_BOARDS 2
	//number of boards supported by the driver
#define MAX_CHANNELS 8
	//max number of channels supported per board

int interrupt_number = 10;
	//the interrupt number used by the boards --
	// can be one of 3, 4, 5, 7, 10, 11, 12, 15
int nboards = 1;	//number of boards installed -- default to 1
ushort base_address[MAX_BOARDS] = {0x308, 0x310};
	//default board addresses

extern uchar fip_firm[];	//the downloaded firmware

int board_inited[8] = {0, 0, 0, 0, 0, 0, 0, 0};
	//whether or not we've "dos_init"ed the board
int open_count[MAX_BOARDS][MAX_CHANNELS];
	//to keep track of open/close nesting
int first_open = 0;
	//first open enables the interrupt system

char o_nowait[MAX_BOARDS][MAX_CHANNELS];
	//0 = must wait for all requested output to go (block), 1 = just output
	// what you can for now (no block)

char i_nowait[MAX_BOARDS][MAX_CHANNELS];
	//0 = must wait for all requested input to come (block), 1 = just input
	// what you can for now (no block)

#define STACKSIZE 100
	//number of words of stack for me

ushort staque[STACKSIZE+1];	//stack for use during the driver

ushort ss_save;	//saver for stack segment on entry
ushort sp_save;	//saver for stack pointer on entry

typedef struct {
	uchar total_byte_count;	//length in bytes of the request header
							// plus any data at end of the request header
	uchar dummy1;			//not used for character devices
	uchar command_code;		//the command code
	ushort status;			//status
	uchar reserved[8];		//reserved for DOS
	uchar dummy2;			//not used for character devices
	ushort end_offset;		//ending address of resident program code
	ushort end_segment;		//ditto
	ushort arg_offset;		//pointer to argument string
	ushort arg_segment;		//ditto
} header_init;

typedef struct {
	uchar total_byte_count;	//length in bytes of the request header
							// plus any data at end of the request header
	uchar dummy1;			//not used for character devices
	uchar command_code;		//the command code
	ushort status;			//status
	uchar reserved[8];		//reserved for DOS
	uchar dummy2;			//not used for character devices
	ushort xfer_offset;		//transfer address
	ushort xfer_segment;	//ditto
	ushort byte_count;		//byte count
} header_io;

typedef struct {
	uchar total_byte_count;	//length in bytes of the request header
							// plus any data at end of the request header
	uchar dummy1;			//not used for character devices
	uchar command_code;		//the command code
	ushort status;			//status
	uchar reserved[8];		//reserved for DOS
	uchar byte_read;		//byte read from the device
} header_ino;
	
typedef struct {
	uchar total_byte_count;	//length in bytes of the request header
							// plus any data at end of the request header
	uchar dummy1;			//not used for character devices
	uchar command_code;		//the command code
	ushort status;			//status
	uchar reserved[8];		//reserved for DOS
} header_stat;				//status, flush, open, close

union {
	header_init far *rhinit;
	header_io far *rhio;
	header_ino far *rhino;
	header_stat far *rhstat;
} rh;
	
char report1[] = "     X board";		//for displays in init
char report2[] = ", address 0xXXXX, ";
char report3[] = "interrupt XX\r\n";

int asserter = 1;	//0=don't, 1=assert dtr & rts on open, de-assert on close

//function prototypes

void dos_strategy (void);
void dos_interrupt00 (void);
void dos_interrupt01 (void);
void dos_interrupt02 (void);
void dos_interrupt03 (void);
void dos_interrupt04 (void);
void dos_interrupt05 (void);
void dos_interrupt06 (void);
void dos_interrupt07 (void);
void dos_interrupt64 (void);
void dos_interrupt65 (void);
void dos_interrupt66 (void);
void dos_interrupt67 (void);
void dos_interrupt68 (void);
void dos_interrupt69 (void);
void dos_interrupt70 (void);
void dos_interrupt71 (void);
void dos_interrupter (int, int);

int dos_close (int, int);
int dos_decode_parameters (void);
int dos_init (int);
int dos_input (int, int);
int dos_input_flush (int, int);
int dos_input_nowait (int, int);
int dos_input_status (int, int);
int dos_ioctl_input (int, int);
int dos_ioctl_output (int, int);
int dos_open (int, int);
int dos_output (int, int);
int dos_output_flush (int, int);
int dos_output_nowait (int, int);
int dos_output_status (int, int);

//macros for "strategy" & "interrupt" routines -- prologue & epilogue

#define SAVE \
	asm { \
		pusha;				/* save machine state */ \
		push ds; \
		push es; \
		pushf; \
		push cs;			/* set the ds to the cs (tiny model!) */ \
		pop ds; \
		mov ss_save,ss;		/* save current stack */ \
		mov sp_save,sp; \
		mov ax,cs;			/* set up a new stack */ \
		mov ss,ax; \
		mov sp,offset staque+STACKSIZE*2; \
	}

#define RESTORE \
	asm { \
		mov ss,ss_save;		/* restore the stack -- MUST BE IN THIS ORDER */ \
		mov sp,sp_save; \
		popf;				/* restore machine state */ \
		pop es; \
		pop ds; \
		popa; \
		pop bp;				/* calling prolog pushes the bp */ \
		db 0cbh;			/* far return */ \
	}

//macros to turn interrupts off, and restore previous interrupt state

#define INTERRUPTS_OFF \
	asm { \
		pushf; \
		cli; \
	}

#define INTERRUPTS_RESTORED \
	asm { \
		popf; \
	}

#define MAX_COMMAND_NUMBER 59
	//currently, the largest command number we recognize

#define MAX_COMMAND_WIDTH 9
	//max number of bytes in a command

char nbytes_per_command[MAX_COMMAND_NUMBER+1] = {
	255, 255, 1, 1, 1, 1, 1, 2, 2, 2,		//0-9
	2, 2, 2, 1, 255, 255, 2, 2, 1, 1,		//10-19
	1, 1, 1, 1, 1, 1, 2, 1, 255, 255,		//20-29
	1, 1, 1, 1, 2, 2, 2, 255, 255, 255,		//30-39
	255, 255, 2, 2, 2, 255, 255, 9, 1, 1,	//40-49
	1, 1, 1, 3, 3, 3, 2, 1, 1, 2			//50-59
};

void dos_strategy ()
{
	//save the request header pointer

	SAVE
	rh.rhinit = (header_init far *) (((long) _ES << 16L) + (long) _BX);
	RESTORE
}

void dos_interrupt00 ()
{
	SAVE
	dos_interrupter (0, 0);	//do the work for this channel
	RESTORE
}

void dos_interrupt01 ()
{
	SAVE
	dos_interrupter (0, 1);	//do the work for this channel
	RESTORE
}

void dos_interrupt02 ()
{
	SAVE
	dos_interrupter (0, 2);	//do the work for this channel
	RESTORE
}

void dos_interrupt03 ()
{
	SAVE
	dos_interrupter (0, 3);	//do the work for this channel
	RESTORE
}

void dos_interrupt04 ()
{
	SAVE
	dos_interrupter (0, 4);	//do the work for this channel
	RESTORE
}

void dos_interrupt05 ()
{
	SAVE
	dos_interrupter (0, 5);	//do the work for this channel
	RESTORE
}

void dos_interrupt06 ()
{
	SAVE
	dos_interrupter (0, 6);	//do the work for this channel
	RESTORE
}

void dos_interrupt07 ()
{
	SAVE
	dos_interrupter (0, 7);	//do the work for this channel
	RESTORE
}

void dos_interrupt64 ()
{
	SAVE
	dos_interrupter (1, 0);	//do the work for this channel
	RESTORE
}

void dos_interrupt65 ()
{
	SAVE
	dos_interrupter (1, 1);	//do the work for this channel
	RESTORE
}

void dos_interrupt66 ()
{
	SAVE
	dos_interrupter (1, 2);	//do the work for this channel
	RESTORE
}

void dos_interrupt67 ()
{
	SAVE
	dos_interrupter (1, 3);	//do the work for this channel
	RESTORE
}

void dos_interrupt68 ()
{
	SAVE
	dos_interrupter (1, 4);	//do the work for this channel
	RESTORE
}

void dos_interrupt69 ()
{
	SAVE
	dos_interrupter (1, 5);	//do the work for this channel
	RESTORE
}

void dos_interrupt70 ()
{
	SAVE
	dos_interrupter (1, 6);	//do the work for this channel
	RESTORE
}

void dos_interrupt71 ()
{
	SAVE
	dos_interrupter (1, 7);	//do the work for this channel
	RESTORE
}

void dos_interrupter (bd, ch)
int bd;	//the board number
int ch;	//the channel number within the board
{
	ushort temp;
	int i;

	switch (rh.rhinit->command_code)
	{
		case 0:	//init -- only do once per board
			if (board_inited[bd] == 0 && bd < nboards)
			//if (board_inited[bd] == 0)
			{
				board_inited[bd] = 1;
				temp = dos_init (0);
				for (i=0; i<MAX_CHANNELS; i++)
					open_count[bd][i] = 0;
			}
			else
				temp = dos_init (1);	//just return end address
			break;
		case 3:	//ioctl input
			if (bd < nboards)
				temp = dos_ioctl_input (bd, ch);
			else
				temp = 0x8101;
			break;
		case 4:	//input
			if (bd < nboards)
				temp = dos_input (bd, ch);
			else
				temp = 0x8101;
			break;
		case 5:	//input non-destructive nowait
			if (bd < nboards)
				temp = dos_input_nowait (bd, ch);
			else
				temp = 0x8101;
			break;
		case 6:	//input status
			if (bd < nboards)
				temp = dos_input_status (bd, ch);
			else
				temp = 0x8101;
			break;
		case 7:	//input flush
			if (bd < nboards)
				temp = dos_input_flush (bd, ch);
			else
				temp = 0x8101;
			break;
		case 8:	//output
		case 9:	//output with verify
			if (bd < nboards)
				temp = dos_output (bd, ch);
			else
				temp = 0x8101;
			break;
		case 10:	//output status
			if (bd < nboards)
				temp = dos_output_status (bd, ch);
			else
				temp = 0x8101;
			break;
		case 11:	//output flush
			if (bd < nboards)
				temp = dos_output_flush (bd, ch);
			else
				temp = 0x8101;
			break;
		case 12:	//ioctl output
			if (bd < nboards)
				temp = dos_ioctl_output (bd, ch);
			else
				temp = 0x8101;
			break;
		case 13:	//open -- asserts dtr & rts
			if (bd < nboards)
				temp = dos_open (bd, ch);
			else
				temp = 0x8101;
			break;
		case 14:	//close -- deasserts dtr & rts
			if (bd < nboards)
				temp = dos_close (bd, ch);
			else
				temp = 0x8101;
			break;
		case 16:	//output nowait
			if (bd < nboards)
				temp = dos_output_nowait (bd, ch);
			else
				temp = 0x8101;
			break;
		default:	//unused
			temp = 0x8103;
	}
	rh.rhinit->status = temp;
	return;
}

int dos_close (bd, ch)
int bd;	//board
int ch;	//channel
{
	if (--open_count[bd][ch] == 0)	//undo the "open" commands
	{
		if (asserter)	//drop dtr & rts
		{
			while (i2_sync_command (bd, ch, (uchar *)"\x03", 1))
				;
			while (i2_sync_command (bd, ch, (uchar *)"\x05", 1))
				;
		}
		while (i2_sync_command (bd, ch, (uchar *)"\x13", 1))	//no more dss
			;
		while (i2_sync_command (bd, ch, (uchar *)"\x15", 1))
			;
		while (i2_sync_command (bd, ch, (uchar *)"\x1B", 1))
			//no more breaks
			;
	}
	return 0x0100;
}

int dos_decode_parameters ()
{
	//decodes config.sys parameters
	//returns 1 if an error detected

	char far *parameters;
	char kar;
	char error;
	int i;
	ushort io;
	ushort irq;
	int ioset;
	int irqset;

	parameters = (char far *) (((rh.rhinit->arg_segment + 0L) << 16L)
		| (rh.rhinit->arg_offset + 0L));

	while (*parameters++ != 0x20)	//skip past first space, which brings
									// to the option area
		;

	error = 0;	//no errors yet
	ioset = 0;	//no io address specified
	irqset = 0;	//no irq specified
	while ((kar = *parameters++) != 0xa)	//terminator
	{
		if (kar == 0x20)
			continue;
		if (kar == 0xd || kar == 0)		//other terminators
			break;
		if (kar == 'B')		//beginning of base=
		{
			if (*parameters++ != 'A')
			{
				error = 1;
				break;
			}
			if (*parameters++ != 'S')
			{
				error = 1;
				break;
			}
			if (*parameters++ != 'E')
			{
				error = 1;
				break;
			}
			if (*parameters++ != '=')
			{
				error = 1;
				break;
			}
			io = 0;	//the built-up i/o address
			ioset = 1;
			for (i=0; i<4; i++)	//get exactly 4 hex digits
			{
				kar = *parameters++;
				if (kar < '0' || kar > 'F')
				{
					error = 1;
					break;
				}
				if (kar > '9' && kar < 'A')
				{
					error = 1;
					break;
				}
				if (kar < 'A')
					io = (io << 4) | (kar - '0');
				else
					io = (io << 4) | (kar - 'A' + 10);
			}
			continue;
		}
		if (kar == 'I')		//beginning of irq=
		{
			if (*parameters++ != 'R')
			{
				error = 1;
				break;
			}
			if (*parameters++ != 'Q')
			{
				error = 1;
				break;
			}
			if (*parameters++ != '=')
			{
				error = 1;
				break;
			}
			irq = 0;	//the built-up i/o address
			irqset = 1;
			for (i=0; i<2; i++)	//get exactly 2 decimal digits
			{
				kar = *parameters++;
				if (kar < '0' || kar > '9')
				{
					error = 1;
					break;
				}
				irq = (irq * 10) + (kar - '0');
			}
			continue;
		}
		if (kar == 'N')		//beginning of assert
		{
			if (*parameters++ != 'O')
			{
				error = 1;
				break;
			}
			if (*parameters++ != 'A')
			{
				error = 1;
				break;
			}
			if (*parameters++ != 'S')
			{
				error = 1;
				break;
			}
			if (*parameters++ != 'S')
			{
				error = 1;
				break;
			}
			if (*parameters++ != 'E')
			{
				error = 1;
				break;
			}
			if (*parameters++ != 'R')
			{
				error = 1;
				break;
			}
			if (*parameters++ != 'T')
			{
				error = 1;
				break;
			}
			asserter = 0;
			continue;
		}
	}
	if (error)
		return 1;
	nboards = 1;
	//now see if they make any sense
	if (ioset)
	{
		if (io & 7)	//must be multiple of 8
			return 1;
		if (io > 0x3f8)	//too high
			return 1;
		base_address[0] = io;
	}
	else
		base_address[0] = 0x308;	//default
	if (irqset)
	{
		if (irq != 3 && irq != 4 && irq != 5 && irq != 7 && irq != 10
			&& irq != 11 && irq != 12 && irq != 15)
			return 1;
		interrupt_number = irq;
	}
	else
		interrupt_number = 10;

	return 0;
}

int dos_init (arg)
int arg;	//0 = normal init, 1 = only send end address of the driver
{
	int i, j, k;
	uchar *pnt;
	char string[4];

	if (arg == 0)
	{
		xputs ("\r\nIntelliPort-II DOS Driver -- Version ");
		xputs (VERSION);
		xputs (", lib ");
		xputs (i2_version);
		xputs (", firmware ");
		string[3] = 0;
		for (i=4; i<=6; i++)
		{
			string[0] = fip_firm[i] / 10 + '0';
			string[1] = fip_firm[i] % 10 + '0';
			if (i != 6)
				string[2] = '.';
			else
				string[2] = 0;
			xputs (string);
		}
		xputs ("\r\n");
		xputs ("Copyright (C) 1991 Computone Corporation -- all rights reserved\r\n");

		if (dos_decode_parameters ())
		{
			xputs ("     Error in config.sys command string for IntelliPort-II\r\n");
			xputs ("     Defaulting to 1 board, address 0x308, interrupt 10\r\n");
			nboards = 1;
			interrupt_number = 10;
			base_address[0] = 0x308;
		}
		else
		{
			report1[5] = nboards + '0';
			xputs (report1);
			if (nboards > 1)
				xputs ("s");
			for (i=0; i<nboards; i++)
			{
				k = base_address[i];
				for (j=15; j>=12; j--)
				{
					report2[j] = (((k & 0xf) <= 9) ? ((k & 0xf) + '0')
						: ((k & 0xf) - 10 + 'A'));
					k >>= 4;
				}
				xputs (report2);
			}
			if (interrupt_number < 10)
			{
				report3[10] = ' ';
				report3[11] = interrupt_number + '0';
			}
			else
			{
				report3[10] = '1';	//relies on numbers < 20
				report3[11] = interrupt_number % 10 + '0';
			}
			xputs (report3);
		}

		i2_lib_init ();	//initialize the library

		if (i2_set_address (0, base_address[0]))
			return 0x810c;
		if (i2_set_address (1, base_address[1]))
			return 0x810c;
		if (i2_set_irq_level (interrupt_number))
			return 0x810c;
		if (i2_set_number_boards (nboards))
			return 0x810c;

		for (i=0; i<nboards; i++)
		{
			if (i2_download_first (i, fip_firm, 1))
				return 0x810c;

			pnt = &fip_firm[512];
			for (j=0; j<fip_firm[1]; j++)
			{
				if (i2_download_second (i, pnt))
					return 0x810c;
				pnt += 512;
			}

			if (i2_download_finish (i))
				return 0x810c;
		}

		if (i2_link_interrupt ())	//link in the interrupt handler
			return 0x810c;

		i2_delay (100);	//small delay
		if (first_open == 0)
		{
			first_open = 1;
			for (i=0; i<nboards; i++)
				i2_enable_interrupts (i);
					//enable the board's interrupting capability
		}
	}

	//return end address of the driver
	//NOTE: here we recover all of the space used by the downloaded code

	rh.rhinit->end_offset = (ushort) &fip_firm;
	rh.rhinit->end_segment = _DS;

	return 0x0100;	//normal return
}

int dos_input (bd, ch)
int bd;	//board
int ch;	//channel
{
	char far *transfer_address;
	int this_time;		//how many done in one shot
	int left_to_do;		//how many left to do

	transfer_address = (char far *) (((rh.rhio->xfer_segment+0L) << 16L)
		| rh.rhio->xfer_offset);

	if (i_nowait[bd][ch])	//no wait, just try to input as much as possible
	{
		rh.rhio->byte_count = i2_input (bd, ch, transfer_address,
			rh.rhio->byte_count);
		return 0x0100;	//done without error
	}

	//here we must input everything

	left_to_do = rh.rhio->byte_count;

	while (left_to_do)
	{
		this_time = i2_input (bd, ch, transfer_address, left_to_do);
		left_to_do -= this_time;
		transfer_address += this_time;
	}

	return 0x0100;	//done without error
}

int dos_input_flush (bd, ch)
int bd;	//board
int ch;	//channel
{
	i2_input_flush (bd, ch);	//flush the input buffer
	return 0x0100;
}

int dos_input_nowait (bd, ch)
int bd;	//board
int ch;	//channel
{
	uchar temp;	//temp
	
	temp = rh.rhino->byte_read;
	if (i2_input_lookahead (bd, ch, &temp))
		return 0;
	else
		return 0x0200;
}

int dos_input_status (bd, ch)
int bd;	//board
int ch;	//channel
{
	return ((i2_input_available (bd, ch)) ? 0 : 0x0200);
}

int dos_ioctl_input (bd, ch)
int bd;	//board
int ch;	//channel
{
	ushort far *transfer_address;
	ushort dss;
	ushort breaks;

	transfer_address = (ushort far *) (((rh.rhio->xfer_segment+0L) << 16L)
		| rh.rhio->xfer_offset);

	if (rh.rhio->byte_count < 4)
		return 0x810c;
	i2_get_status (bd, ch, &dss, &breaks);
	*transfer_address++ = dss;
	*transfer_address++ = breaks;
	return 0x0100;
}

int dos_ioctl_output (bd, ch)
int bd;	//board
int ch;	//channel
{
	char far *transfer_address;
	int left_to_do;		//how many left to do
	uchar command_block[MAX_COMMAND_WIDTH];	//buffer for a command
	int i;

	transfer_address = (char far *) (((rh.rhio->xfer_segment+0L) << 16L)
		| rh.rhio->xfer_offset);

	//special trap for commands internal to the driver that don't
	// go to the board (setting & clearing wait/nowait flags)

	if (*transfer_address == -1)
	{
		switch (*++transfer_address)
		{
			case 0:
				i_nowait[bd][ch] = 0;
				break;
			case 1:
				i_nowait[bd][ch] = 1;
				break;
			case 2:
				o_nowait[bd][ch] = 0;
				break;
			case 3:
				o_nowait[bd][ch] = 1;
				break;
		}
		return 0x0100;
	}

	left_to_do = rh.rhio->byte_count;

	while (left_to_do)
	{
		i = *transfer_address;	//get command code
		if (i > MAX_COMMAND_NUMBER || i < 0)
			return 0x8103;
		i = nbytes_per_command[i];
		if (i == 255)	//don't do these commands!
			return 0x8103;
		if (i > MAX_COMMAND_WIDTH)
			return 0x810a;
		i2_movedata (FP_SEG (transfer_address), FP_OFF (transfer_address),
			_DS, (unsigned) command_block, i);
		while (i2_sync_command (bd, ch, command_block, i))
			;
		transfer_address += i;
		left_to_do -= i;
	}

	return 0x0100;	//done without error
}

int dos_open (bd, ch)
int bd;	//board
int ch;	//channel
{
	if (asserter)
	{
		while (i2_sync_command (bd, ch, (uchar *)"\x02", 1))	//assert dtr
			;
		while (i2_sync_command (bd, ch, (uchar *)"\x04", 1))	//assert rts
			;
	}

	while (i2_sync_command (bd, ch, (uchar *)"\x12", 1))	//see cts changes
		;
	while (i2_sync_command (bd, ch, (uchar *)"\x14", 1))	//see dcd changes
		;
	while (i2_sync_command (bd, ch, (uchar *)"\x1A\x00", 2))	//report breaks
		;
	open_count[bd][ch]++;
	return 0x0100;
}

int dos_output (bd, ch)
int bd;	//board
int ch;	//channel
{
	char far *transfer_address;
	int this_time;		//how many done in one shot
	int left_to_do;		//how many left to do

	transfer_address = (char far *) (((rh.rhio->xfer_segment+0L) << 16L)
		| rh.rhio->xfer_offset);

	if (o_nowait[bd][ch])	//no wait, just try to output as much as possible
	{
		rh.rhio->byte_count = i2_output (bd, ch, transfer_address,
			rh.rhio->byte_count);
		return 0x0100;	//done without error
	}

	//here we must output everything

	left_to_do = rh.rhio->byte_count;

	while (left_to_do)
	{
		this_time = i2_output (bd, ch, transfer_address, left_to_do);
		left_to_do -= this_time;
		transfer_address += this_time;
	}

	return 0x0100;	//done without error
}

int dos_output_flush (bd, ch)
int bd;	//board
int ch;	//channel
{
	i2_output_flush (bd, ch);
	return 0x0100;
}

int dos_output_nowait (bd, ch)
int bd;	//board
int ch;	//channel
{
	char save;
	int returner;

	save = o_nowait[bd][ch];
	o_nowait[bd][ch] = 1;
	returner = dos_output (bd, ch);
	o_nowait[bd][ch] = save;
	return returner;
}

int dos_output_status (bd, ch)
int bd;	//board
int ch;	//channel
{
	return ((i2_output_free (bd, ch) == 0) ? 0x0200 : 0);
}

void xdebug (string, val)
char *string;
int val;
{
	int i;
	char string2[2];

	xputs (string);
	string2[1] = 0;
	i = 4;
	while (i--)
	{
		string2[0] = (val >> 12) & 0xf;
		if (string2[0] < 10)
			string2[0] += '0';
		else
			string2[0] += 'A' - 10;
		val <<= 4;
		xputs (string2);
	}
	xputs (";");
	return;
}

void xputs (string)
char *string;
{
	//output a string to the console

	uchar page;
	uchar attr;

	_AH = 0xf;				//get current video page
	asm {
		int 010h;
	}
	page = _BH;
	
	_BH = page;				//get current attribute
	_AH = 8;
	asm {
		int 010h;
	}
	attr = _AH;

	while (*string)
	{
		_BL = attr;
		_AL = *string++;
		_AH = 0xe;
		asm {
			int 010h;
		}
	}
	return;
}
