//i2test.c -- test program to output data to all channels of
// an IntelliPort-II board, using i2lib

#define VERSION "0.01"

extern unsigned _stklen = 8192;

//0.01 -- 4/30/91 -- first release

#include <stdio.h>
#include <stdlib.h>
#include <alloc.h>
#include <dos.h>
#include <mem.h>
#include <conio.h>

typedef unsigned short ushort;
typedef unsigned char uchar;

#include "i2glob.h"
	//the function prototypes

#include "fip_firm.h"
	//the firmware to download

#define BSIZE 80
	//number of bytes to malloc in each "o_line" & etc. (below)

static char far *o_line[8];
	//malloc'ed buffers for output

static char far *i_line[8];
	//malloc'ed buffers for input

static char far *i_check[8];
	//malloc'ed buffers for input checking

static int error;
static int do_unlink = 0;
static int mallocs_done = 0;

static int control_c_got = 0;

static unsigned long outter[8] = {0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L};
	//count of characters output per channel

static unsigned long inner[8] = {0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L};
	//count of characters input per channel

static int o_offset[8] = {0, 0, 0, 0, 0, 0, 0, 0};
	//current offset into the output string, for each channel

static int i_offset[8] = {0, 0, 0, 0, 0, 0, 0, 0};
	//current offset into the input checking string, for each channel

static int error_count[8] = {0, 0, 0, 0, 0, 0, 0, 0};
	//input error count for each channel

void main (void);
void fatal_error (char *);
int breaker (void);

void main ()
{
	int i, j, k;
	uchar *pnt;
	int n;

	ctrlbrk (breaker);	//control-c handler

	//malloc a small array for each channel

	for (i=0; i<8; i++)
	{
		o_line[i] = (char far *)farmalloc (BSIZE);
		if (o_line[i] == NULL)
		{
bad_malloc:
			printf ("\n\007Error malloc'ing -- totally weird\n");
			exit (1);
		}
		o_line[i][0] = i + '0';
		o_line[i][1] = '/';
		for (j=2; j<8; j++)
			o_line[i][j] = '0';	//line number
		o_line[i][8] = '-';
		for (j=9; j<BSIZE; j++)
			o_line[i][j] = j - 9 + 0x21;
		i_line[i] = (char far *)farmalloc (BSIZE);
		if (i_line[i] == NULL)
			goto bad_malloc;
		i_check[i] = (char far *)farmalloc (BSIZE);
		if (i_check[i] == NULL)
			goto bad_malloc;
		_fmemcpy (i_check[i], o_line[i], BSIZE);
	}
	mallocs_done = 1;

	i2_lib_init ();	//initialize the library

	error = i2_set_address (0, 0x308);	//set the board 0 address to 308
	if (error)
		fatal_error ("i2_set_address");

	error = i2_set_irq_level (10);		//use interrupt 10
	if (error)
		fatal_error ("i2_set_irq_level");

	error = i2_set_number_boards (1);	//only 1 board will be used
	if (error)
		fatal_error ("i2_set_number_boards");

	error = i2_download_first (0, fip_firm, 1);
	if (error)
		fatal_error ("i2_download_first");

	pnt = &fip_firm[512];
	for (i=0; i<fip_firm[1]; i++)
	{
		error = i2_download_second (0, pnt);
		if (error)
		{
			printf ("\n\007Error on block # %d -- ", i);
			fatal_error ("i2_download_second");
		}
		pnt += 512;
	}

	error = i2_download_finish (0);
	if (error)
		fatal_error ("i2_download_finish");

	error = i2_link_interrupt ();	//link in the interrupt handler
	if (error)
		fatal_error ("i2_link_interrupt");
	do_unlink = 1;

	delay (100);	//small delay
	i2_enable_interrupts (0);	//enable the board's interrupting capability

	for (k=0; k<8; k++)	//assert dtr & rts for each channel
	{
		while (i2_sync_command (0, k, (uchar *)"\002", 1))	//assert dtr
			;
		while (i2_sync_command (0, k, (uchar *)"\004", 1))	//assert rts
			;
		while (i2_sync_command (0, k, (uchar *)"\x07\x15", 2))	//64000 baud
			;
	}

	//delay (5000);

	while (1)
	{
		if (control_c_got)
			break;
		if (kbhit ())
		{
			getch ();
			break;
		}
		for (i=0; i<8; i++)	//first, do some output
		{
			n = i2_output (0, i, &o_line[i][o_offset[i]], BSIZE-o_offset[i]);
			o_offset[i] += n;
			if (o_offset[i] >= BSIZE)	//hit the end -- bump linenumber
			{
				for (j=7; j>=2; j--)
				{
					if (++o_line[i][j] <= '9')
						break;
					o_line[i][j] = '0';
				}
				o_offset[i] = 0;	// and reset the offset
			}
			outter[i] += n;
		}
		for (i=0; i<8; i++)	//now do some input
		{
			n = i2_input (0, i, &i_line[i][i_offset[i]], BSIZE-i_offset[i]);
			i_offset[i] += n;
			if (i_offset[i] >= BSIZE)	//hit the end -- check the data,
									// then bump linenumber
			{
				if (_fmemcmp (i_line[i], i_check[i], BSIZE))
				{
					printf ("?");
					error_count[i]++;
				}
				for (j=7; j>=2; j--)
				{
					if (++i_check[i][j] <= '9')
						break;
					i_check[i][j] = '0';
				}
				i_offset[i] = 0;	// and reset the offset
			}
			inner[i] += n;
		}
	}

doner:
	do_unlink = 0;
	error = i2_unlink_interrupt ();
	if (error)
		fatal_error ("i2_unlink_interrupt");
	error = 0;
	fatal_error ("");
}

void fatal_error (string)
char *string;
{
	int error2;
	int i;

	if (do_unlink)
	{
		error2 = i2_unlink_interrupt ();
		if (error2)
			printf ("\n\007Error unlinking interrupts -- %d\n", error2);
	}
	if (error)
		printf ("\n\007%s -- %d\n", string, error);
	if (mallocs_done)
	{
		for (i=0; i<8; i++)
		{
			farfree (o_line[i]);
			farfree (i_line[i]);
			farfree (i_check[i]);
		}
	}
	for (i=0; i<8; i++)
		printf ("channel %d:  output: %.8lu  input: %.8lu  input errors: %d\n", i, outter[i], inner[i], error_count[i]);
	exit (error || error2);
}

int breaker ()
{
	control_c_got = 1;
	return 1;	//to make the compiler happy
}
