page	60,132
title	(C) 1991 Computone Corporation -- intt.asm

;******************************************************************************
;*                                                                            *
;*                                                                            *
;*                                                                            *
;*                  (C) Copyright 1991 Computone Corporation                  *
;*                                                                            *
;*          Unauthorized use, duplication, or disclosure prohibited           *
;*                                                                            *
;*              This module contains PROPRIETARY INFORMATION of               *
;*                            Computone Corporation                           *
;*                   and should be treated as CONFIDENTIAL.                   *
;*                                                                            *
;*                                                                            *
;*                                                                            *
;******************************************************************************

;------------------------------------------
; intt.asm
; To support the low-level interrupt test;
; Waits for incoming data (low byte only)
; Each time a byte is received, the board will
; wait for its outbound mailbox to be clear,
; then select a new interrupt to use (according
; to the data received) and then write to the
; outbound mailbox.  Because the host test
; code has enabled mailbox interrupts, this
; causes an interrupt on the appropriate line.
;------------------------------------------

VERSION		equ	1
REVISION	equ	10
SUBREVISION	equ	2

;-----------------------------------------------------------------------
; Revision History:
;
;1.0.0 -- 6/6/91 -- first edition
;1.10.1 -- 7/27/91 -- Will delay until mailbox is clear
;		after hitting mailbox interrupt. before looking for next
;		interrupt request. (host will read the mailbox at his
;		interrupt level.
;-----------------------------------------------------------------------

include lw_h1.inc

;------------------------------------
; Table to convert user's interrupt
; request to a value for IRQ select
; register
;------------------------------------

itable label byte
db	0, 0, 0, SEL_I3, SEL_I4, SEL_I5, 0, SEL_I7
db	0, 0, SEL_I10, SEL_I11, SEL_I12, 0, 0, SEL_I15
db	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
db	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
db	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
db	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
db	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
db	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
db	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
db	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
db	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
db	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
db	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
db	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
db	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
db	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
db	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0

	; long-term register use:
	; cl = color of led
	; ch = operating mode (expandable or not)
	; bx = points to translation table

initialization:

	xor	ax, ax
	mov	ds, ax
	GET_OPERATING_MODE

	mov	ch, al		; Keep operating mode in ch

	cli
	cld

	mov	ax, cs
	mov	ds, ax
	
	ASSUME ds:_TEXT, cs:_TEXT

	mov	al, GREEN	;turn LED green
	mov	dx, LED_IRQ
	out	dx, al
	mov	cl, al		; store current color
	mov	bx, offset itable

	;interrupt test
	;no stack used

wait_1:	
	;----------------------------
	; Wait for incoming FIFO data
	;----------------------------
	mov	dx, FIFO_CMD
	in	al, dx
	test	al, EMPT
	jnz	wait_1

	;-----------------------------
	; Read the data (low byte only)
	;-----------------------------
	mov	dx, FIFO_DATA
	in	al, dx	;read it -- it is the interrupt number

	xlatb		; look up the value to use

	;----------------------------
	; Select interrupt type
	;----------------------------

	xor	cl, RED		; To toggle LED (green, yellow...)
	or	al, cl		; blend color bits with irq selections
	mov	dx, LED_IRQ	;
	out	dx, al		; Hit the selection register.
	
	;---------------------------------
	; Write mailbox to cause interrupt
	;---------------------------------

	HIT_MAILBOX		ch, 10h	; (an arbitrary value)

	;-------------------------------------------
	; Wait for host to read the mailbox
	; (i.e., to service the interrupt) before
	; proceeding to the next interrupt.
	;-------------------------------------------
	
	WAIT_FOR_MB_READ	ch	; (ch has operating mode)

	jmp wait_1
	
include lw_h2.inc

