subttl lw_h1.inc -- Standard header for diagnostic loadware
page
;------------------------------------------
; lw_h1.inc
;
; Header file for diagnostic loadware
;
; Contains register defines, Low-ram
; definitions, macros, etc...
; 
; (cosmetic note: expects tabstops=8)
;------------------------------------------

;-----------------------------------------------
; Revision history:
;	10/29/91	Split from individual files
;-----------------------------------------------

	.186

	.LALL

;-----------------------------
; 186 internal peripherals
;-----------------------------
;timer registers

TMR0COUNT	equ	0ff50h
TMR0MAXA	equ	0ff52h
TMR0MAXB	equ	0ff54h
TMR0MODE	equ	0ff56h

TMR1COUNT	equ	0ff58h
TMR1MAXA	equ	0ff5ah
TMR1MAXB	equ	0ff5ch
TMR1MODE	equ	0ff5eh

TMR2COUNT	equ	0ff60h
TMR2MAXA	equ	0ff62h
TMR2MODE	equ	0ff66h

;interrupt control registers

INT3CONTROL	equ	0ff3eh
INT2CONTROL	equ	0ff3ch
INT1CONTROL	equ	0ff3ah
INT0CONTROL	equ	0ff38h
INTDMA1		equ	0ff36h
INTDMA0		equ	0ff34h
INTTIMER	equ	0ff32h
MASKREGISTER	equ	0ff28h
EOIREGISTER	equ	0ff22h

;dma registers	(Supported on IntelliPort-IIEX only: 
;		channel 1 <-- fifo input
;		channel 0 --> fifo output


DMA0_SOURCEL	equ	0ffc0h	; Source address, low & high
DMA0_SOURCEH	equ	0ffc2h
DMA0_DESTL	equ	0ffc4h	; Destination address, low & high
DMA0_DESTH	equ	0ffc6h
DMA0_COUNT	equ	0ffc8h
DMA0_CTRL	equ	0ffcah	; Control word: bits below--

DMA1_SOURCEL	equ	0ffd0h	; Source address, low & high
DMA1_SOURCEH	equ	0ffd2h
DMA1_DESTL	equ	0ffd4h	; Destination address, low & high
DMA1_DESTH	equ	0ffd6h
DMA1_COUNT	equ	0ffd8h
DMA1_CTRL	equ	0ffdah	; Control word: bits below--

	; List of bits for DMA control word (Not complete)
	DC_WORD		equ	1	; word/byte transfer
	DC_BYTE		equ	0

	DC_START	equ	6	; Change and start both set
	DC_STOP		equ	4	; Change set, start clear

	DC_PRI		equ	20h	; Set to give priority
	DC_SOURCE_SYNC	equ	40h	; Source synchronized
	DC_DEST_SYNC	equ	80h	; Destination synchronized

	DC_TERMINATE	equ	200h	; Terminate on terminal count
	DC_SINC		equ	400h	; Increment source pointer
	DC_SMEM		equ	1000h	; Source is in memory space
	DC_DINC		equ	2000h	; Increment destination pointer
	DC_DMEM		equ	8000h	; Destination is in memory space



page
;--------------------------------
;definition of CD1400 registers
;--------------------------------

GFRCR	equ	40h * 2
CAR	equ	68h * 2
SVRR	equ	67h * 2
RIR	equ	6bh * 2
TIR	equ	6ah * 2
MIR	equ	69h * 2
PPR	equ	7eh * 2

RIVR	equ	43h * 2
TDR	equ	63h * 2
RDSR	equ	62h * 2
MISR	equ	4ch * 2

LIVR	equ	18h * 2
CCR	equ	05h * 2
SRER	equ	06h * 2
COR1	equ	08h * 2
COR2	equ	09h * 2
COR3	equ	0ah * 2
COR4	equ	1eh * 2
COR5	equ	1fh * 2
RDCR	equ	0eh * 2
SCHR1	equ	1ah * 2
SCHR2	equ	1bh * 2
SCHR3	equ	1ch * 2
SCHR4	equ	1dh * 2
MCOR1	equ	15h * 2
MCOR2	equ	16h * 2
RTPR	equ	21h * 2
MSVR1	equ	6ch * 2
MSVR2	equ	6dh * 2
RBPR	equ	78h * 2
RCOR	equ	7ch * 2
TBPR	equ	72h * 2
TCOR	equ	76h * 2

MAPPER	equ	0feh
		;address to get at the chip mapper

;the led and irq register
LED_IRQ		equ	0100h


;values for LED_IRQ
RED		equ	080h
GREEN		equ	040h
YELLOW		equ	RED or GREEN
BLACK		equ	000h
IRQ_ENABLE	equ	020h
SEL_I3		equ 20h
SEL_I4		equ 24h
SEL_I5		equ 28h
SEL_I7		equ 2ch
SEL_I10		equ 30h
SEL_I11		equ 34h
SEL_I12		equ 38h
SEL_I15		equ 3ch

FIFO_DATA	equ	0104h
	;the bi-fifo or regular fifo data port

FIFO_CMD	equ	0106h
	;the bi-fifo command/status port
	;the expandable fifo status port

FIFO_MBX	equ	0108h
	;for the expandable, the mailbox port

READ_IN		equ	01000000b
	;value to write to the IRQEN (expandable) to read inbound mailbox

; Status flags for expandable and non-expandable
EMPT		equ	20h	;inbound fifo is empty
MB		equ	04h	;mailbox data available

HAS_OUTMAIL	equ	01h	; outbound mailbox full (expandable only)
				; (Bit in a different position from on
				;  the host size).

;bi-fifo pointer numbers
FIFO_COMMAND	equ	1
FIFO_MASK	equ	2
FIFO_OUTMAILBOX	equ	4
FIFO_INMAILBOX	equ	6
FIFO_RESETWRITE	equ	7

;Debug-port registers

DEBUG_BASE		equ	200h

D2692_MRA		equ	DEBUG_BASE + 0
D2692_CRA		equ	DEBUG_BASE + 4
D2692_MRB		equ	DEBUG_BASE + 16
D2692_CRB		equ	DEBUG_BASE + 20

subttl lw_h1.inc -- Define fields in low DRAM
page
_data0	segment public 'data'

	;interrupt vectors go here

	org 0

vectors	dd 64 dup (?)	;the interrupt vectors

	org 100h
	
	;the next 16 bytes are the 16 bytes thrown in the fifo
	; at power-up

bmagic1	db	?	;magic number byte 1 -- should be 096h
bmagic2	db	?	;magic number byte 2 -- should be 035h
bver	db	?	;bootup firmware version number
brev	db	?	;bootup firmware revision number
bsubrev	db	?	;bootup firmware sub-revision number
bprodid	db	?	;product id
bbus	db	?	;bus type
bmsize	db	?	;memory size
bport	db	?	;number of ports -- for 4- & 8-port products
				;for expandable:
				;	bit 0:	1 = 1400 #0 present
				;	bit 1:	1 = 1400 #1 present
				;	bit 2:	1 = 1400 #2 present
				;	bit 3:	1 = 1400 #3 present
				;	bit 4:	1 = 1400 #4 present
				;	bit 5:	1 = 1400 #5 present
				;	bit 6:	1 = 1400 #6 present
				;	bit 7:	1 = 1400 #7 present
bdiag1	db	?	;diagnostic result 1
bdiag2	db	?	;diagnostic result 2
bspeed	db	?	;cpu speed
bcpu	db	?	;cpu type
bex14	db	?	;unused for 4- & 8-port products
				;for expandable:
				;	bit 0:	1 = 1400 #8 present
				;	bit 1:	1 = 1400 #9 present
				;	bit 2:	1 = 1400 #10 present
				;	bit 3:	1 = 1400 #11 present
				;	bit 4:	1 = 1400 #12 present
				;	bit 5:	1 = 1400 #13 present
				;	bit 6:	1 = 1400 #14 present
				;	bit 7:	1 = 1400 #15 present
bfsize	db	?	;unused for 4- & 8-port products
				;for expandable:
				;	log base 2 size of fifo, each direction
				;	e.g.:	8: 256-byte fifo, each direction
				;			9: 512
				;			10: 1024
				;			11: 2048
				;			12: 4096
				;			etc.
bnbox	db	?	;unused for 4- & 8-port products
				;for expandable:
				;	number of boxes attached (0-4)

	;the next 4 bytes are the download jumper

bdown	dd	?	;the long pointer -- not used
bblock	dw	?	;number of blocks in the downloaded code

;NOTE: these first 22 bytes must be exactly in this spot (100h-115h)
if ($-offset vectors) ne 116h
	.ERR	;locations 110h-115h of low memory are untouchable !!
endif

half_second	dw ?
	;used for fatal flashes timing
irq_select	db ?
if ($-offset vectors) ne 119h
	.ERR	;half-second timer must be at 116h
		; and irq_select must be at 118h
endif

		db	?	;for alignment
_data0	ends

subttl lw_h1.inc -- Useful macros
page

IS_EXPANDABLE	equ 1
IS_16		equ 2

;----------------------------------------------------------
; GET_OPERATING_MODE
; Puts a map of IS_16 and IS_EXPANDABLE in the AL register
; (assumes ds = 0)
;----------------------------------------------------------

GET_OPERATING_MODE macro
local t1

	xor	al, al
	mov	ah, bprodid	; Look at product id
	and	ah, 0c0h
	cmp	ah, 040h
	jne	t1
	or	al, IS_EXPANDABLE
	mov	ah, bbus
	and	ah, 030h
	cmp	ah, 030h
	jne	t1
	or	al, IS_16
t1:
	xor	ah, ah
endm	

;-------------------------------------
; WAIT_FOR_MAILBOX_IN
; Waits for the incoming mailbox to
; have something in it.
; Then reads that something: returns it in al
; mode has whether expandable or not.
;-------------------------------------

WAIT_FOR_MAILBOX_IN	macro mode
local t1, t2, t3

	mov	dx, FIFO_CMD
t1:
	in	al, dx
	test	al, MB
	jz	t1

	test	mode, IS_EXPANDABLE
	jnz	t2

	mov	al, FIFO_INMAILBOX
	out	dx, al
	in	al, dx
	jmp	t3
t2:
	mov	dx, FIFO_MBX
	in	al, dx
t3:
endm

;-------------------------------------
; WRITE_MASK_VALUE mode, value
; Writes value to the mask 
; register.  mode has whether expandable
; or non-expandable
;-------------------------------------
WRITE_MASK_VALUE	macro	mode, value
local t1, t2

	mov	dx, FIFO_CMD
	test	mode, IS_EXPANDABLE
	jnz	t1

	mov	al, FIFO_MASK
	out	dx, al
	mov	al, value
	out	dx, al
	jmp	t2
t1:
	mov	al, value
	out	dx, al
t2:
endm

;-----------------------------------------
; READ_DATA_WORD	mode
;
; Waits for data to be available, then
; reads an entire word (two bytes at a time
; if appropriate.)
;-----------------------------------------


READ_DATA_WORD	macro	mode
local t1, t1a, t2, t3

	mov	dx, FIFO_CMD
t1:
	in	al, dx
	test	al, EMPT
	jnz	t1

	mov	dx, FIFO_DATA
	test	mode, IS_16
	jnz	t2

	in	al, dx		; 8-bit mode, read lsb first.
	mov	ah, al

	mov	dx, FIFO_CMD
t1a:
	in	al, dx		; wait in-between bytes
	test	al, EMPT
	jnz	t1a

	mov	dx, FIFO_DATA
	in	al, dx
	xchg	al, ah
	jmp	t3
t2:
	in	ax, dx
t3:
endm

;-----------------------------------------
; WRITE_DATA_WORD	mode
;
; Assumes room is available in output fifo.
; writes an entire word (in two passes if needed).
;-----------------------------------------


WRITE_DATA_WORD	macro	mode
local t1, t2

	mov	dx, FIFO_DATA
	test	mode, IS_16
	jnz	t1

	out	dx, al
	mov	al, ah
	out	dx, al
	jmp	t2
t1:
	out	dx, ax
t2:
endm

	
;-------------------------------------------
; HIT_MAILBOX mode value
; Writes the value to the output mailbox
; mode has whether expandable or non-expandable
;-------------------------------------------

HIT_MAILBOX	macro	mode, value
local t1, t2

	test	mode, IS_EXPANDABLE
	jnz	t1

	mov	dx, FIFO_CMD
	mov	al, FIFO_OUTMAILBOX
	out	dx, al
	mov	al, value
	out	dx, al
	jmp	t2
t1:
	mov	dx, FIFO_MBX
	mov	al, value
	out	dx, al
t2:

endm

;-------------------------------------------
; WAIT_FOR_MB_READ mode
; waits until outgoing mailbox is empty
; mode has whether expandable or non-expandable
;-------------------------------------------

WAIT_FOR_MB_READ	macro mode
local t1, t2, t3

	mov	dx, FIFO_CMD
	test	mode, IS_EXPANDABLE
	jnz	t2
t1:
	mov	al, FIFO_OUTMAILBOX
	out	dx, al
	in	al, dx
	test	al, al
	jnz	t1
	jmp	t3
t2:
	in	al, dx
	test	al, HAS_OUTMAIL
	jnz	t2
t3:

endm

;---------------------------------
; TOGGLE_LED value
; value = value ^ RED
; LED <- value
; Usually used to GREEN <-> YELLOW
;----------------------------------

TOGGLE_LED	macro value

	mov	al, value
	xor	al, RED
	mov	dx, LED_IRQ
	out	dx, al
	mov	value, al

endm

;-------------------------------------
; DEAD_RED
; Turn the LED red and loop in place.
;-------------------------------------	

DEAD_RED	macro
local t1
	mov	al, RED
	mov	dx, LED_IRQ
	out	dx, al
	cli
t1:
	jmp	t1
endm

subttl lw_h1.inc -- Start definition of text space
page
_TEXT	segment para public 'CODE'

	assume cs:_TEXT, ds:_data0, es:nothing, ss:nothing

	org 0

;NOTE: following 16 bytes, plus initial instruction, plus text string, must
; occur exactly here, i.e., at beginning of code space
	
header:
	db 0	;filled in by conversion program -- magic number
	db 0	;filled in by conversion program -- number of ADDITIONAL
			; 512-byte blocks coming
	db 0	;lsbits of crc -- filled in by conversion program
	db 0	;msbits of crc -- filled in by conversion program
	db VERSION
	db REVISION
	db SUBREVISION
	db 9 dup (0)	;currently unused
	db 32 dup (0)	;for time, etc.
	jmp initialization	;first instruction executed upon download
	db "BeGiNnInG oF cOdE"


;----------------------------------------------
; End of this file: user code should begin here:
; (using the label "initialization")
;-----------------------------------------------
subttl Main code: initialization
page

