page	60,132
title	(C) 1991 Computone Corporation -- memr.asm

;******************************************************************************
;*                                                                            *
;*                                                                            *
;*                                                                            *
;*                  (C) Copyright 1991 Computone Corporation                  *
;*                                                                            *
;*          Unauthorized use, duplication, or disclosure prohibited           *
;*                                                                            *
;*              This module contains PROPRIETARY INFORMATION of               *
;*                            Computone Corporation                           *
;*                   and should be treated as CONFIDENTIAL.                   *
;*                                                                            *
;*                                                                            *
;*                                                                            *
;******************************************************************************

;-----------------------------------------
; memr.asm
;
; Random-pattern memory test
;-----------------------------------------


VERSION		equ	1
REVISION	equ	0
SUBREVISION	equ	1

;----------------------------------------------------------
; Revision History
;
;1.0.0 -- 6/6/91 -- first edition
;1.0.1	-- 10/30/91	MAG	Revised to support II/IIEX
;----------------------------------------------------------

include lw_h1.inc


BIG_PRIME	equ	076c3h ;big prime number

;the following are copied from low memory
memsize		db 0

;other variables

npasses		dw 0
color		db 0
start_value	dw 0
adder		dw 0
mode		db 0

initialization:

	mov	al,GREEN	; Led starts green
	mov	dx,LED_IRQ
	out	dx,al

	cli
	cld
	sub	ax,ax		;get low memory variables we care about
	mov	ds,ax
	mov	al,bmsize
	mov	cs:memsize,al

	GET_OPERATING_MODE
	mov	cs:mode, al

	mov ax,cs		;set all seg registers to the code space
	mov ds,ax
	mov es,ax
	mov ss,ax

	;random memory diagnostic
	;no stack used

	;first, get number of passes

	READ_DATA_WORD	mode
	mov	npasses, ax

	mov	al,memsize	;shove the size out the fifo
	out	dx,al
	
	mov color,GREEN
	mov start_value,1
	mov adder, BIG_PRIME

main_loop:
	;test a multiple of 512-byte blocks, beginning at memory address
	; 0, and ending at the beginning of downloaded code

	; Increment each value by the adder (which varies from pass to pass)
	; between each 512-block, complement ax (!)

	mov dx,start_value	;starting value to write for this pass
	mov ax,dx	;starting value to dump
	mov bp, adder

	sub bx,bx	;starting segment

ram_l2:
	mov es,bx	;set segment
	mov cx,128	;number of words to dump/2 (unwound)
	sub di,di	;where to dump

ram_l3:
	stosw
	add ax,bp	;a big prime number
	stosw
	add ax,bp	;a big prime number
	loop ram_l3

	not ax		; additional complement between 512-byte sections

	add bx,32	;bump the segment
	mov cx,cs	;check for beginning of code
	cmp bx,cx
	jb ram_l2

	;done writing -- now check the data

	mov dx,	start_value
	mov di,	dx
	sub bx,bx	;starting segment

ram_l4:
	mov es,bx
	mov cx,256
	sub si,si

ram_l5:
	mov	ax,es:[si]	; previous version cleared memory as it went.
					; now why would that be better?
	add si,2
	cmp ax,di
	jne mr_fail
	add di,bp
	loop ram_l5

	not	di

	add bx,32
	mov cx,cs
	cmp bx,cx
	jb ram_l4

	;pass complete without error

	TOGGLE_LED	color

	add start_value,487	;new starting value (bumped by prime)
	mov	ax, start_value
	ror ax, 8
	add adder, ax		; Bump the adder by another amount

	mov dx,FIFO_DATA
	sub al,al			;output 0 code to fifo
	out dx,al
	dec npasses
	jz done_all
	jmp main_loop

done_all:
	jmp done_all		;just hang

mr_fail:
	mov	dx,FIFO_DATA
	mov	ss,ax			;data gotten back

	mov	al,1			;error code
	out	dx,al

	mov	ax,es			;send the segment
	WRITE_DATA_WORD mode

	mov ax,si			;send the offset
	sub ax,2
	WRITE_DATA_WORD	mode

	mov ax,di			;data written
	WRITE_DATA_WORD	mode

	mov ax,ss			;data read back
	WRITE_DATA_WORD	mode

	DEAD_RED
	
include lw_h2.inc	

