page	60,132
title	(C) 1991 Computone Corporation -- memw.asm

;******************************************************************************
;*                                                                            *
;*                                                                            *
;*                                                                            *
;*                  (C) Copyright 1991 Computone Corporation                  *
;*                                                                            *
;*          Unauthorized use, duplication, or disclosure prohibited           *
;*                                                                            *
;*              This module contains PROPRIETARY INFORMATION of               *
;*                            Computone Corporation                           *
;*                   and should be treated as CONFIDENTIAL.                   *
;*                                                                            *
;*                                                                            *
;*                                                                            *
;******************************************************************************

;-----------------------------------------------------
; memw.asm
; Walking-bit memory test
;
;-----------------------------------------------------
VERSION		equ	1
REVISION	equ	0
SUBREVISION	equ	1

;-----------------------------------------------------
; Revision History
;
;1.0.0 -- 6/6/91 -- first edition
;1.0.1 -- 10/30/81	MAG	Revised for II/IIEX support
;-----------------------------------------------------

include lw_h1.inc

;the following are copied from low memory
memsize		db 0

;other variables

npasses		dw 0
color		db 0
blockwords	dw 0
blocksegs	dw 0
mode		db 0

initialization:

	mov al,GREEN
	mov dx,LED_IRQ
	out dx,al

	cli
	cld
	sub	ax, ax		;get low memory variables we care about
	mov	ds, ax
	mov	al, bmsize
	mov	cs:memsize, al

	GET_OPERATING_MODE
	mov	cs:mode, al

	mov ax,cs		;set all seg registers to the code space
	mov ds,ax
	mov es,ax
	mov ss,ax

	;walking bit memory diagnostic
	;no stack used

	;first, get number of passes

	READ_DATA_WORD	mode
	mov	npasses, ax

	mov	al,memsize	;shove the size out the fifo
	out	dx,al		; (byte operation: not native)
	
	mov	color,GREEN

	mov	blockwords, 65536/2
	mov	blocksegs, 65536/16
	jmp	main_loop

fail_1:
	sub	si,2
	mov	di,0
	jmp	mr_fail

fail_2:
	mov	di,0
	jmp	mr_fail

fail_3:
	mov	di,dx
	jmp	mr_fail


main_loop:
	;test a multiple of 512-byte blocks, beginning at memory address
	; 0, and ending at the beginning of downloaded code

	sub	bx,bx	;starting segment

ram_l2:
	mov	es,bx	;set segment

	mov	cx,blockwords	;number of words to work on at a time
	sub	di,di	;where to dump
	sub	ax,ax
	rep	stosw	;clear to zeros

	mov	cx,blockwords	;check for zeros
	sub	si,si

walk_1:
	mov	ax,es:[si]
	add	si,2
	or	ax,ax
	jnz	fail_1
	loop	walk_1

	mov	cx,blockwords	;now, for each location, walk the bit
	sub	si,si

walk_2:
	mov	ax,es:[si]
	or	ax,ax		;must start as 0
	jne	fail_2
	mov	dx,1		;the bit to walk

walk_3:			; the loop is un-wound to save loops...
	mov	word ptr es:[si],dx
	jmp	short $+2
	mov	ax,es:[si]
	cmp	ax,dx
	jne	fail_3
	shl	dx,1
	mov	word ptr es:[si],dx
	jmp	short $+2
	mov	ax,es:[si]
	cmp	ax,dx
	jne	fail_3
	shl	dx,1
	jnz	walk_3

	mov	word ptr es:[si],0ffffh	;throw in all ones at end
	add	si,2
	loop	walk_2

	mov	cx,blockwords	;now check for all ones
	sub	si,si

walk_4:
	mov	ax,es:[si]
	add	si,2
	cmp	ax,0ffffh
	jne	fail_4
	loop	walk_4
	
	TOGGLE_LED	color

	add	bx,blocksegs	;bump to next segment
	mov	cx,bx
	add	cx,blocksegs	;add amount of ram to test (= size of segment)
	mov	ax,cs
	cmp	cx,ax			;compare IT to beginning location of code.
	ja	end_pass
	jmp	ram_l2

end_pass:
	;pass complete without error

	; Each pass to do smaller & smaller blocks

	cmp	blocksegs, 16
	je	freeze_size

	shr	blocksegs, 1
	shr	blockwords, 1

freeze_size:

	mov	dx,FIFO_DATA
	sub	al,al			;output 0 code to fifo
	out	dx,al			;(as a byte: not native mode)
	dec	npasses
	jz	done_all
	jmp	main_loop

done_all:
	jmp	done_all		;just hang when complete.


fail_4:
	sub	si,2
	mov	di,0ffffh
	
mr_fail:
	mov	dx,FIFO_DATA
	mov	ss,ax			;data gotten back
	mov	al,1			;error code
	out	dx,al

	mov	ax,es			;send the segment
	WRITE_DATA_WORD	mode

	mov	ax,si			;send the offset
	WRITE_DATA_WORD mode

	mov	ax,di			;data written
	WRITE_DATA_WORD	mode
	
	mov	ax,ss			;data read back
	WRITE_DATA_WORD	mode

	DEAD_RED

include lw_h2.inc	

