/*---------------------
 * pci.c
 * Diagnostic to determine
 * a motherboard's level
 * of PCI compliance.
 *----------------------


/*-------------------------------------
 * Cosmetic note:
 * Written using tabstops at 4 spaces
 * vi: set ts=4
 * pr: -e4
 *-------------------------------------
 */

/*-------------------------------------------------------
 * Revision History:
 *
 *  5/22/96		MAG	First Draft
 *-------------------------------------------------------
 */

#define VERSION "0.0.1"

#include <stdio.h>
#include <stdlib.h>
#include <alloc.h>
#include <dos.h>
#include <mem.h>
#include <conio.h>
#include <sys\stat.h>
#include <io.h>
#include <time.h>
#include <string.h>
#include <fcntl.h>



/*-------------------------
 * MAIN
 *-------------------------
 */

main(argc, argv)
int argc; 
char **argv;
{
int bus;
int vers;
int mech;

	if (!biosPCIpresent())
	{
		printf("PCI BIOS not was detected\n");
		return 1;
	}
	printf("PCI BIOS detected.\n");
	if ((bus = biosPCIbus()) != -1)
	{
		printf("Last bus number(0-based)= %d\n", bus);
	}
	if ((vers = biosPCIvers()) != -1)
	{
		printf("Bios Major Version %d, Minor Version %d\n",
			((vers >> 8) & 0xff), (vers & 0xff) );
	}
	if ((mech = biosPCImech()) != -1)
	{
		if(mech & 1) printf("Supports Config Mechanism #1\n");
		if(mech & 2) printf("Supports Config Mechanism #2\n");
		if(mech & 0x10) printf("Special Cycles via Mechanism #1\n");
		if(mech & 0x20) printf("Special Cycles via Mechanism #2\n");
	}
}

/*-------------------------------
 * PCI support is after this part.
 *-------------------------------
 */

asm .386

static int
getPCIhandle(int index)
{
int retval;

	asm {
		mov ax, 0b102h;
		mov cx, 0291h;
		mov dx, 8e0eh;
		mov si, index;
		int 1ah;
		jc bbpci1;
		cmp ah, 0;
		jne bbpci1;
		mov retval, bx;
	}
	return retval;
bbpci1:
	return -1;
}

static int
getPCIword(int handle, int addr)
{
int retval;

	asm {
		mov ax, 0b109h;
		mov	bx, handle;
		mov di, addr;
		int 1ah;
		mov retval, cx;
	}
	return retval;
}

static unsigned char
getPCIbyte(int handle, int addr)
{
unsigned char retval;

	asm {
		mov ax, 0b108h;
		mov	bx, handle;
		mov di, addr;
		int 1ah;
		mov retval, cl;
	}
	return retval;
}

		
/*-----------------
 * biosPCIpresent()
 * Returns True if
 * BIOS is detected, 
 * False otherwise.
 *-----------------
 */

int
biosPCIpresent()
{
	asm {
		mov ax, 0b101h;
		int 1ah;
		jc  bbpci0;
		cmp edx, 20494350h;
		jne bbpci0;
		cmp ah, 0;
		jne bbpci0;
		}
		return 1;
bbpci0:
		return 0;
}
	


int
biosPCImech()	/* Returns bit-mapped PCI mechanism */
				/* Returns -1 if PCI Bios not not found */
{
unsigned int retval;

	asm {
		mov ax, 0b101h;
		int 1ah;
		jc  bbpci1;
		cmp edx, 20494350h;
		jne bbpci1;
		cmp ah, 0;
		jne bbpci1;
		mov retval, ax
		}
		return retval;
bbpci1:
		return -1;
}
	

int
biosPCIvers()	/* Returns major & minor versions */
				/* Returns -1 if PCI Bios not not found */
{
unsigned int retval;

	asm {
		mov ax, 0b101h;
		int 1ah;
		jc  bbpci2;
		cmp edx, 20494350h;
		jne bbpci2;
		cmp ah, 0;
		jne bbpci2;
		mov retval, bx
		}
		return retval;
bbpci2:
		return -1;
}
	

int
biosPCIbus()	/* Returns number of last bus */
				/* Returns -1 if PCI Bios not not found */
{
unsigned int retval;

	asm {
		mov ax, 0b101h;
		int 1ah;
		jc  bbpci3;
		cmp edx, 20494350h;
		jne bbpci3;
		cmp ah, 0;
		jne bbpci3;
		xor ch, ch
		mov retval, cx
		}
		return retval;
bbpci3:
		return -1;
}
	
#ifdef NOT_DEFINED

findPCIslot(int * paddress, int * pirq)
{
int handle;
int goodness = 0;
int setup_address[8];
int setup_irq[8];
int i;
int index;

	for(index = 0; index < 8; index++)
	{
		if((handle=getPCIhandle(index)) == -1) break;

		/* We found a PCI card... */

		setup_address[index] = getPCIword(handle, 0x14) & 0xfffe;
		setup_irq[index] = getPCIbyte(handle, 0x3c);

		if (*paddress == 0)
		{
			*paddress = setup_address[index];
			*pirq = setup_irq[index];
		}
		if (*paddress == setup_address[index])
		{
			Amcc_Address = getPCIword(handle, 0x10) & 0xfffe;
			goodness = 1;
		}
		if (setup_irq[index] == *pirq || !*pirq)
		{
			goodness |= 2;
		}
	}
	if(mainVerbosity & V_STARTUP || goodness != 3)
	{
		for(i = 0; i < index; i++)
		{
			printf("Computone PCI board %d at address 0x%x, interrupt %d\n",
				i, setup_address[i], setup_irq[i]);
		}
	}
	if (goodness != 3)
	{
		printf("PCI Bios: no board configured for ");
		if (*paddress)
		{
			printf("address 0x%x, ", *paddress);
		}
		else
		{
			printf("any address, ");
		}
		if (*pirq)
		{
			printf("irq %d\n");
		}
		else
		{
			printf("\n");
		}
	}
	return goodness == 3;	
}

/*--------------------------------------
 * pci_checkout(f)
 * 
 * Checks out some operation registers of
 * the AMCC chip on a PCI card.
 *
 * Input parameters:
 * 		f_passes:	number of passes (0 --> 65536)
 * Output:
 * 		f_error:	OKAY:			no error
 *
 *		f_error:	BAD_POS_2		pos register 2 failed
 *		f_error:	BAD_POS_3		pos register 3 failed
 *		f_wstring[0]	data written
 *		f_rstring[0]	data read back
 *		f_error:	BAD_MC			not seemingly on a microchannel
 *-------------------------------------------------------------
 */
			
int
pci_checkout (pfunk f)
{
int i, j;
int passes;

	/* Note that this test has the interrupts off almost the
	 * whole time: implication is that ctrl-C cannot break us
	 * out.
	 */

	if (!hasUsedB || (busType != PCI_OPT) || !Amcc_Address)
	{
		POST_ERROR(f, BAD_PCI);
	}
	
	passes = f->f_passes;
	f->f_passfail = 0;	//spin through the passes
	do {
			for(i = 0; i < 256; i++)
			{
				/* Yes I know I'm only writing bytes. That is
				 * why my loop stops after 255.
				 */
				OUTB(Amcc_Address +0x0, i);
				OUTB(Amcc_Address +0x4, i ^ 0x55);
				OUTB(Amcc_Address +0x8, i ^ 0xaa);
				OUTB(Amcc_Address +0xc, i ^ 0xff);
				
				j = INB(Amcc_Address + 0x0);
				if (i != j) 
				{	
					f->f_results[0] = Amcc_Address + 0x0;
					f->f_wstring[0] = i;
					f->f_wstring[1] = j;
					POST_ERROR(f, BAD_OMB);
				}
				
				j = INB(Amcc_Address + 0x4);
				if ((i ^ 0x55) != j) 
				{	
					f->f_results[0] = Amcc_Address + 0x4;
					f->f_wstring[0] = i ^ 0x55;
					f->f_wstring[1] = j;
					POST_ERROR(f, BAD_OMB);
				}
				
				j = INB(Amcc_Address + 0x8);
				if ((i ^ 0xaa) != j) 
				{	
					f->f_results[0] = Amcc_Address + 0x8;
					f->f_wstring[0] = i ^ 0xaa;
					f->f_wstring[1] = j;
					POST_ERROR(f, BAD_OMB);
				}
				
				j = INB(Amcc_Address + 0xc);
				if ((i ^ 0xff) != j) 
				{	
					f->f_results[0] = Amcc_Address + 0xc;
					f->f_wstring[0] = i ^ 0xff;
					f->f_wstring[1] = j;
					POST_ERROR(f, BAD_OMB);
				}
		}
	
		f->f_passfail++;
	} while (--passes);
	
	SUCCEED(f);
}
#endif /* NOT_DEFINED */
