page	60,132
title	(C) 1997 Computone Corporation -- regt3.asm

;******************************************************************************
;*                                                                            *
;*                                                                            *
;*                                                                            *
;*                  (C) Copyright 1997 Computone Corporation                  *
;*                                                                            *
;*          Unauthorized use, duplication, or disclosure prohibited           *
;*                                                                            *
;*              This module contains PROPRIETARY INFORMATION of               *
;*                            Computone Corporation                           *
;*                   and should be treated as CONFIDENTIAL.                   *
;*                                                                            *
;*                                                                            *
;*                                                                            *
;******************************************************************************

VERSION		equ	1
REVISION	equ	2
SUBREVISION	equ	1
	;version number information

;1.0.0 -- 7/23/92 -- first edition (derived from regt.asm)
;1.1.0 -- 1/21/97 -- Starting changes to support IntelliPort-III
;			hardware.
;1.2.0 -- 3/06/98 -- ARR special cases added.
;1.2.1 -- 6/23/99 -- ...and another (15H)

include lw_h1.inc



;-----------------------------------
; Values used to return error codes
; when there is a failure
;-----------------------------------


DEVICE_1400	equ	1
	GLOBAL_REG	equ	1
	CHANNEL_REG	equ	2
DEVICE_8259	equ	2
DEVICE_80186	equ	3
DEVICE_2692	equ	4
DEVICE_MAPPER	equ	5
DEVICE_LCD	equ	6


page

;------------------------------------
; TABLE OF MAPPER VALUES
;-----------------------------------

BOX_1234	equ	28h	; selects the last box
BOX_123		equ	8h	; selects the next-last box
BOX_12		equ	18h	; selects the next-next-last box
BOX_1		equ	10h	; selects the first box when there's four

; for 8259's,

maps_8259_4	db	BOX_1 or 4
maps_8259_3	db	BOX_12 or 4
maps_8259_2	db	BOX_123 or 4
maps_8259_1	db	BOX_1234 or 4

;for Async resource register

maps_resource_4	db BOX_1 or 5
maps_resource_3	db BOX_12 or 5
maps_resource_2	db BOX_123 or 5
maps_resource_1	db BOX_1234 or 5

;for LCD register
maps_lcd_4	db BOX_1 or 7
maps_lcd_3	db BOX_12 or 7
maps_lcd_2	db BOX_123 or 7
maps_lcd_1	db BOX_1234 or 7

; for 1400's, 

maps_1400_4	label	byte	; if there are 4 boxes, start here
db	BOX_1 or 0
db	BOX_1 or 1
db	BOX_1 or 2
db	BOX_1 or 3
maps_1400_3	label	byte	; if there are 3 boxes, start here
db	BOX_12 or 0
db	BOX_12 or 1
db	BOX_12 or 2
db	BOX_12 or 3
maps_1400_2	label	byte	; if there are 2 boxes, start here
db	BOX_123 or 0
db	BOX_123 or 1
db	BOX_123 or 2
db	BOX_123 or 3
maps_1400_1	label	byte	; if there is one box, start here
db	BOX_1234 or 0
db	BOX_1234 or 1
db	BOX_1234 or 2
db	BOX_1234 or 3


box_count_to_offset	label word
dw	offset maps_1400_1
dw	offset maps_1400_2
dw	offset maps_1400_3
dw	offset maps_1400_4

box_count_to_8259_offset label word
dw offset maps_8259_1
dw offset maps_8259_2
dw offset maps_8259_3
dw offset maps_8259_4

box_count_to_resource_offset label word
dw offset maps_resource_1
dw offset maps_resource_2
dw offset maps_resource_3
dw offset maps_resource_4	

box_count_to_lcd_offset label word
dw offset maps_lcd_1
dw offset maps_lcd_2
dw offset maps_lcd_3
dw offset maps_lcd_4

page
;-- copied from low memory (bootstrap firmware info)

product_id	db 0	; if non-expandable, indicates isa-4, isa-8

number_of_boxes	db 0
box_types	db 0	; Bit-mapped (set for 16-port box, clear for 8-port)
					; 1 << n for box n (0-3)

isa_cex4	db 0	; Set if this is an ISA-4 (asic)

;other variables

value			db 5Ah
color			db 0
mapper			db 0
mapper_mask		db 1	; defaults for isa-8
mode			db 0
which_1400		db 0
which_8259		db 0
which_lcd		db 0
which_resource	db 0


; particularly for expandable

map_1400	dw	0		; where does map start?
map_8259	dw	0
map_lcd		dw	0
map_resource	dw 0

uart_limit	db	0
pic_limit	db	0
lcd_limit	db  0
resource_limit	db 0

exists_1400	label byte
db	1, 1, 1, 1
db	1, 1, 1, 1
db 	1, 1, 1, 1
db 	1, 1, 1, 1

is_654		label	byte	; Byte is set if UART is
				; supposed to be an ST654.
db	0, 0, 0, 0
db	0, 0, 0, 0
db	0, 0, 0, 0
db	0, 0, 0, 0

no_pic	label	byte		; Byte is set if the box
db	0, 0, 0, 0		; uses 654's (therefore not
				; using an 8259-compatible PIC
				

even
type_and_address	dw	0	; lsb has CHANNEL_REG or GLOBAL_REG
					; msb has io_address to test.

STACKSIZE	equ	100
stacker		dw STACKSIZE dup (?)

initialization:

	cli
	cld
	mov	al,GREEN	; LED green initially
	mov	dx,LED_IRQ
	out	dx,al

	sub	ax,ax		;get low memory variables we care about
	mov	ds,ax

	GET_OPERATING_MODE
	mov	cs:mode, al

	mov	al, bprodid
	mov	cs:product_id, al

	mov	al, bnbox
	mov	cs:number_of_boxes, al

	mov	al, bcpu
	shr	al, 4
	mov	cs:box_types, al

	mov	al, bcpu
	and	al, 4
	mov	cs:isa_cex4, al	; Indicate if this is an ISA-4 (asic)

	mov	ax,cs		;set all seg registers to the code space
	mov	ds,ax
	mov	es,ax
	mov	ss,ax
	mov	sp,offset stacker+STACKSIZE*2

	assume ds:_TEXT, cs:_TEXT

	; Before starting the diagnostic, make sure all the timers
	; are turned off!

	mov	ax, 4000h	; if timer is off, we can read/write 
				; the counts and they don't change.
	mov	dx, TMR0MODE
	out	dx, ax
	mov	dx, TMR1MODE
	out	dx, ax
	mov	dx, TMR2MODE
	out	dx, ax

	;register diagnostic 
	;uses stack

	; The test runs forever...

	mov color,GREEN

	test	mode, IS_EXPANDABLE
	jz	non_expand_support
	jmp	expandable_support

page
non_expand_support:

	mov	al, product_id
	and	al, 3
	cmp	al, 3	; is it an ISA-4?
	jne	main_loop

	mov	mapper_mask, 0	; yes, so only one 1400 to check
	
main_loop:		;-- Main passes start here --

	;-- Test all the 1400's

	mov	bl, value
	ror	bl, 1
	mov	value, bl	; Each pass rotates the value in "value"
	mov	bh, 0		; bh has chip-mapper value
next_1400:
	mov	al, bh
	out	MAPPER, al	; select a (1400) chip
	mov	dx, GFRCR
	mov	al, bl
	out	dx, al		; Write twice, read twice.
	out	dx, al
	in	al, dx
	in	al, dx

	inc	bh
	and	bh, mapper_mask
	jnz	next_1400

	jmp	main_loop


page

expandable_support:

	; Figure out how many boxes we have so we know what
	; mapper values to write.

	; initialize a table which tracks which uarts are supposed to exist.

	xor	cx, cx
	mov	cl, number_of_boxes
	mov	si, offset exists_1400
	mov	ah, box_types
	jcxz	es_2
es_1:
	mov	byte ptr [si], 1 ; for any kind of box, first two 1400's exist
	inc	si

	cmp	isa_cex4, 0	; EXCEPTION: ISA-4 (asic) has only 1 chip
	je	es_1a		; 0 = normal case.

	xor	ah, ah
	mov	byte ptr[si], ah
	jmp	es_1b

es_1a:
	mov	byte ptr [si], 1
es_1b:
	inc	si

	mov	al, ah		; let al == 1 only for 16-port box
	and	al, 1
	mov	[si], al
	inc	si
	mov	[si], al
	inc	si

	shr	ah, 1
	loop	es_1
es_2:

	; Based on number of boxes, initialize start of
	; the mapper table, and calculate the highest uart.
	; (similarly for pic...)

	mov	al, number_of_boxes
	mov	bl, al		; spare copy for index
	mov	pic_limit, al	; time 1 part per box
	mov	lcd_limit, al
	mov	resource_limit, al
	shl	al, 2		; times 4 parts per box
	mov	uart_limit, al

	dec	bx
	and	bx, 3
	shl	bx, 1
	mov	ax, box_count_to_offset[bx]
	mov	map_1400, ax
	mov	ax, box_count_to_8259_offset[bx]
	mov	map_8259, ax
	mov	ax, box_count_to_resource_offset[bx]
	mov	map_resource, ax
	mov	ax, box_count_to_lcd_offset[bx]
	mov	map_lcd, ax

	; Initialize a table which indicates whether each
	; UART is an ST654 or a CD1400
	; (does not concern itself with whether the
	; uart would be there, only what it would be if
	; it were.
	; Also initialize a table which indicates whether there
	; is a PIC or not. (The ST654 products don't use an 8259-compatible
	; PIC device).
	 
	xor	cx, cx
	mov	cl, number_of_boxes
	mov	bx, map_resource
	mov	si, offset is_654
	mov	di, offset no_pic

es_3:	mov	al, [bx]
	out	MAPPER, al	; Map in the resource register
	jmp	short $+2
	in	al, 0		; Read the resource register.

	xor	dx, dx		; assume CD1400 unless otherwise known

	cmp	al, 35h		; Any kind of PowerRack card?
	je	es_3st
	cmp	al, 34h
	je	es_3st
	
	;  Version 1.2.0: More special cases:
	cmp	al, 04h		; 8-port 654 non-expandable
	je	es_3st
	cmp	al, 14h		; 4-port 654 non-expandable (just in case)
	je	es_3st
	cmp	al, 15h		; 4-port 654 non-expandable 10-pin 
	je	es_3st

	test	al, 40h		; Expandable 654's?
	jz	es_3cd		; no
	
es_3st:
	mov	dx, 0101h		; Set to indicate ST654's
es_3cd:
	mov	[di], dl		; Patch no_pic table
	inc	di 
	mov	word ptr[si], dx	; and is_654 table.
	mov	word ptr[si+2], dx
	add	si, 4
	
	inc	bx		; next box
	loop	es_3		; if it exists.

expand_loop:		;-- Main passes start here --


	mov	cl, value
	ror	cl, 1
	mov	value, cl	; Each pass rotates the value in "value"

	;-- Test the 8259's

	xor	ch, ch		; which_8259
	jmp	test_limit_8259

next_8259:
	mov	bl, ch
	xor	bh, bh

	cmp	no_pic[bx], 0
	jne	skip_8259	; Skip boxes without 8259's
	
	add	bx, map_8259
	mov	al, [bx]

	out	MAPPER, al	; select a (1400) chip

	mov	dx, 2
	mov	al, cl
	out	dx, al
	out	dx, al
	in	al, dx
	in	al, dx
	
skip_8259:
	inc	ch

test_limit_8259:
	cmp	ch, pic_limit
	jne	next_8259
	jmp	test_1400e


	;-- Test all the 1400's

test_1400e:
	xor	ch, ch		; start with 0, if not too high!
	jmp	test_limit_1400

next_1400e:
	mov	bl, ch	; convert the uart number into a mapper
	xor	bh, bh
	cmp	exists_1400[bx], 0	; if the 1400 does not exist
	jz	skip_e_test		; on this box, skip

	cmp	is_654[bx], 0	; Should this be a 654?
	jne	doit_654	; Yes, so separate tactics.
	
	add	bx, map_1400
	mov	al, [bx]

	out	MAPPER, al	; select a (1400) chip

	mov	dx, GFRCR
	mov	al, cl
	out	dx, al
	out	dx, al
	in	al, dx
	in	al, dx

skip_e_test:

	inc	ch

test_limit_1400:
	cmp	ch, uart_limit
	jne	next_1400e
	
	jmp	expand_loop

doit_654:
	
	; Adjust chip-mapper value so it is
	; valid for ST654's
	
	mov	ah, bl		; Chip number
	add	bx, map_1400
	mov	al, [bx]

	and	al,  38H	; Box-selection bits only
	and	ah, 2		; select whether first two chips
	shr	ah, 1		; maps to LSB of mapper
	or	al, ah		; al now has correct mapper value

   	out	MAPPER, al	; select a (ST654) chip
     
	; Set DX to the chip base address to use
	
	mov	dl, bl		; start with chip number
	and	dx, 1		; First, third, start at 0,
	shl	dx, 6		; Second, forth, start at 40H
	
	add	dx, 0eh		; Offset of scratch register
	mov	al, 55h

	push	cx
	mov	cx, 4	; 4 uarts per chip.
doit_a:
	out	dx, al
	out	dx, al
	in	al, dx
	in	al, dx
	add	dx, 10h	; Next uart address
	loop	doit_a
	
	pop	cx

	jmp	skip_e_test
	
include lw_h2.inc

